local addonName, MaxDps = ...;

LibStub('AceAddon-3.0'):NewAddon(MaxDps, 'MaxDps', 'AceConsole-3.0', 'AceEvent-3.0', 'AceTimer-3.0');

--- @class MaxDps
_G[addonName] = MaxDps;

local TableInsert = tinsert;
local TableRemove = tremove;
local TableContains = tContains;
local TableIndexOf = tIndexOf;

local UnitIsFriend = UnitIsFriend;
local IsPlayerSpell = IsPlayerSpell;
local UnitClass = UnitClass;
local GetSpecialization = GetSpecialization;
local CreateFrame = CreateFrame;
local GetAddOnInfo = C_AddOns.GetAddOnInfo;
local IsAddOnLoaded = C_AddOns.IsAddOnLoaded
local LoadAddOn = C_AddOns.LoadAddOn;

local spellHistoryBlacklist = {
	[75] = true; -- Auto shot
};

function MaxDps:OnInitialize()
	self.db = LibStub('AceDB-3.0'):New('MaxDpsOptions', self.defaultOptions);

	self:RegisterChatCommand('maxdps', 'ShowMainWindow');

	if not self.db.global.customRotations then
		self.db.global.customRotations = {};
	end

	self:AddToBlizzardOptions();
end

function MaxDps:ShowMainWindow()
	if not self.Window then
		self.Window = self:GetModule('Window');
	end

	self.Window:ShowWindow();
end

function MaxDps:GetTexture()
	if self.db.global.customTexture ~= '' and self.db.global.customTexture ~= nil then
		self.FinalTexture = self.db.global.customTexture;
		return self.FinalTexture;
	end

	self.FinalTexture = self.db.global.texture;
	if self.FinalTexture == '' or self.FinalTexture == nil then
		self.FinalTexture = 'Interface\\Cooldown\\ping4';
	end

	return self.FinalTexture;
end

MaxDps.DefaultPrint = MaxDps.Print;
function MaxDps:Print(message,level)
	if not level then level = "info" end
	if self.db.global.disabledInfo and self.db.global.disabledInfo == "none" then
		return
	elseif self.db.global.disabledInfo and self.db.global.disabledInfo == "all" then
		MaxDps:DefaultPrint(message);
	elseif self.db.global.disabledInfo and self.db.global.disabledInfo == "errorinfo" then
		if level == "error" or level == "info" then
			MaxDps:DefaultPrint(message);
		end
	elseif self.db.global.disabledInfo and self.db.global.disabledInfo == "error" then
		if level == "error" then
			MaxDps:DefaultPrint(message);
		end
	elseif self.db.global.disabledInfo and self.db.global.disabledInfo == "info" then
		if level == "info" then
			MaxDps:DefaultPrint(message);
		end
	end
end

MaxDps.profilerStatus = 0;
function MaxDps:ProfilerStart()
	local profiler = self:GetModule('Profiler');
	profiler:StartProfiler();
	self.profilerStatus = 1;
end

function MaxDps:ProfilerStop()
	local profiler = self:GetModule('Profiler');
	profiler:StopProfiler();
	self.profilerStatus = 0;
end

function MaxDps:ProfilerToggle()
	if self.profilerStatus == 0 then
		self:ProfilerStart();
	else
		self:ProfilerStop();
	end
end

function MaxDps:EnableRotation()
	if self.NextSpell == nil or self.rotationEnabled then
		self:Print(self.Colors.Error .. 'Failed to enable addon!', "error");
		return
	end

	self:Fetch();
	self:UpdateButtonGlow();

	self:CheckTalents();
	self:GetAzeriteTraits();
	self:GetAzeriteEssences();
	self:GetCovenantInfo();
	self:GetLegendaryEffects();
	self:CheckIsPlayerMelee();
	if self.ModuleOnEnable then
		self.ModuleOnEnable();
	end

	self:EnableRotationTimer();

	self.rotationEnabled = true;
end

function MaxDps:EnableRotationTimer()
	self.RotationTimer = self:ScheduleRepeatingTimer('InvokeNextSpell', self.db.global.interval);
end

function MaxDps:DisableRotation()
	if not self.rotationEnabled then
		return
	end

	self:DisableRotationTimer();

	self:DestroyAllOverlays();
	self:Print(self.Colors.Info .. 'Disabling', "info");

	self.Spell = nil;
	self.rotationEnabled = false;
end

function MaxDps:DisableRotationTimer()
	if self.RotationTimer then
		self:CancelTimer(self.RotationTimer);
	end
end

local talentUpdateEvents = {
	"TRAIT_CONFIG_CREATED",
	"ACTIVE_COMBAT_CONFIG_CHANGED",
	"STARTER_BUILD_ACTIVATION_FAILED",
	"PLAYER_TALENT_UPDATE",
	"AZERITE_ESSENCE_ACTIVATED",
	"TRAIT_CONFIG_DELETED",
	"TRAIT_CONFIG_UPDATED",
	"LOADING_SCREEN_DISABLED",
}

function MaxDps:OnEnable()
	self:RegisterEvent('PLAYER_TARGET_CHANGED');
	self:RegisterEvent('PLAYER_REGEN_DISABLED');
	self:RegisterEvent('PLAYER_REGEN_ENABLED');

	for _, event in pairs(talentUpdateEvents) do
		self:RegisterEvent(event, 'TalentsUpdated')
	end

	self:RegisterEvent('ACTIONBAR_SLOT_CHANGED', 'ButtonFetch');
	self:RegisterEvent('ACTIONBAR_HIDEGRID', 'ButtonFetch');
	self:RegisterEvent('ACTIONBAR_PAGE_CHANGED', 'ButtonFetch');
	self:RegisterEvent('ACTIONBAR_UPDATE_STATE', 'ButtonFetch');
	self:RegisterEvent('UPDATE_BONUS_ACTIONBAR', 'ButtonFetch');
	self:RegisterEvent('LEARNED_SPELL_IN_TAB', 'ButtonFetch');
	self:RegisterEvent('CHARACTER_POINTS_CHANGED', 'ButtonFetch');
	self:RegisterEvent('ACTIVE_TALENT_GROUP_CHANGED', 'ButtonFetch');
	self:RegisterEvent('PLAYER_SPECIALIZATION_CHANGED', 'ButtonFetch');
	self:RegisterEvent('UPDATE_MACROS', 'ButtonFetch');
	self:RegisterEvent('VEHICLE_UPDATE', 'ButtonFetch');
	self:RegisterEvent('UPDATE_STEALTH', 'ButtonFetch');
	self:RegisterEvent('SPELLS_CHANGED', 'ButtonFetch');

	self:RegisterEvent('UNIT_ENTERED_VEHICLE');
	self:RegisterEvent('UNIT_EXITED_VEHICLE');

	self:RegisterEvent('NAME_PLATE_UNIT_ADDED');
	self:RegisterEvent('NAME_PLATE_UNIT_REMOVED');
	--	self:RegisterEvent('PLAYER_REGEN_ENABLED');

	if not self.playerUnitFrame then
		self.spellHistory = {};

		self.playerUnitFrame = CreateFrame('Frame');
		self.playerUnitFrame:RegisterUnitEvent('UNIT_SPELLCAST_SUCCEEDED', 'player');
		self.playerUnitFrame:SetScript('OnEvent', function(_, _, _, _, spellId)
			-- event, unit, lineId
			if IsPlayerSpell(spellId) and not spellHistoryBlacklist[spellId] then
				TableInsert(self.spellHistory, 1, spellId);

				if #self.spellHistory > 5 then
					TableRemove(self.spellHistory);
				end
			end
		end);
	end

	self:Print(self.Colors.Info .. 'Initialized', "info");
end

MaxDps.visibleNameplates = {};
function MaxDps:NAME_PLATE_UNIT_ADDED(_, nameplateUnit)
	if not TableContains(self.visibleNameplates, nameplateUnit) then
		TableInsert(self.visibleNameplates, nameplateUnit);
	end
end

function MaxDps:NAME_PLATE_UNIT_REMOVED(_, nameplateUnit)
	local index = TableIndexOf(self.visibleNameplates, nameplateUnit);
	if index ~= nil then
		TableRemove(self.visibleNameplates, index)
	end
end

function MaxDps:TalentsUpdated()
	self:DisableRotation();
	self:UpdateSpellsAndTalents()
end

local MaxDpsSpellTable
function MaxDps:UpdateSpellsAndTalents()
    local idtoclass = {
        [1] = "Warrior",
        [2] = "Paladin",
        [3] = "Hunter",
        [4] = "Rogue",
        [5] = "Priest",
        [6] = "Death Knight",
        [7] = "Shaman",
        [8] = "Mage",
        [9] = "Warlock",
        [10] = "Monk",
        [11] = "Druid",
        [12] = "Demon Hunter",
        [13] = "Evoker",
    }
	local idtospec = {
	    --Death Knight
	    [250] = "Blood",
	    [251] = "Frost",
	    [252] = "Unholy",
	    --Demon Hunter
	    [577] = "Havoc",
	    [581] = "Vengeance",
	    --Druid
	    [102] = "Balance",
	    [103] = "Feral",
	    [104] = "Guardian",
	    [105] = "Restoration",
		--Evoker
		[1473] = "Augmentation",
		[1467] = "Devastation",
		[1468] = "Preservation",
	    --Hunter
	    [253] = "Beast Mastery",
	    [254] = "Marksmanship",
	    [255] = "Survival",
	    --Mage
	    [62] = "Arcane",
	    [63] = "Fire",
	    [64] = "Frost",
	    --Monk
	    [268] = "Brewmaster",
	    [269] = "Windwalker",
	    [270] = "Mistweaver",
	    --Paladin
	    [65] = "Holy",
	    [66] = "Protection",
	    [70] = "Retribution",
	    --Priest
	    [256] = "Discipline",
	    [257] = "Holy",
	    [258] = "Shadow",
	    --Rogue
	    [259] = "Assassination",
	    [260] = "Outlaw",
	    [261] = "Subtlety",
	    --Shaman
	    [262] = "Elemental",
	    [263] = "Enhancement",
	    [264] = "Restoration",
	    --Warlock
	    [265] = "Affliction",
	    [266] = "Demonology",
	    [267] = "Destruction",
	    --Warrior
	    [71] = "Arms",
	    [72] = "Fury",
	    [73] = "Protection",
    }
	MaxDpsSpellTable = {
		["Warrior"] = {
			["Protection"] = {
			["SecondWind"] = 29838,
			["Charge"] = 100,
			["BatteringRam"] = 394312,
			["Uproar"] = 391572,
			["VictoryRush"] = 34428,
			["ArmoredtotheTeeth"] = 394855,
			["BraceForImpact"] = 386030,
			["Riposte"] = 161798,
			["DanceofDeath"] = 393965,
			["LeechingStrikes"] = 382258,
			["PiercingHowl"] = 12323,
			["DisruptingShout"] = 386071,
			["UnnervingFocus"] = 384042,
			["Ravager"] = 228920,
			["SeismicReverberation"] = 382956,
			["DefendersAegis"] = 397103,
			["OneHandedWeaponSpecialization"] = 382895,
			["Revenge"] = 6572,
			["Pummel"] = 6552,
			["HeavyRepercussions"] = 203177,
			["ThunderousWords"] = 384969,
			["FastFootwork"] = 382260,
			["ImpenetrableWall"] = 384072,
			["SuddenDeath"] = 29725,
			["ChampionsSpear"] = 376079,
			["ShieldCharge"] = 385952,
			["Instigate"] = 394311,
			["BestServedCold"] = 202560,
			["Sidearm"] = 384404,
			["ShieldSlam"] = 23922,
			["HeroicThrow"] = 57755,
			["Hamstring"] = 1715,
			["Vanguard"] = 71,
			["FuriousBlows"] = 390354,
			["EnduringAlacrity"] = 384063,
			["Taunt"] = 355,
			["RumblingEarth"] = 275339,
			["LastStand"] = 12975,
			["ToughasNails"] = 385888,
			["ThunderousRoar"] = 384318,
			["PainandGain"] = 382549,
			["UnstoppableForce"] = 275336,
			["DoubleTime"] = 103827,
			["Whirlwind"] = 1680,
			["Devastator"] = 236279,
			["Strategist"] = 384041,
			["Bloodsurge"] = 384361,
			["BarbaricTraining"] = 390675,
			["BerserkerShout"] = 384100,
			["MasteryCriticalBlock"] = 76857,
			["ChampionsMight"] = 386285,
			["ImpendingVictory"] = 202168,
			["ChampionsBulwark"] = 386328,
			["ShieldBlock"] = 2565,
			["IntimidatingShout"] = 5246,
			["Shockwave"] = 46968,
			["FueledbyViolence"] = 383103,
			["Bloodborne"] = 385704,
			["Avatar"] = 401150,
			["Slam"] = 1464,
			["DeepWounds"] = 115768,
			["AngerManagement"] = 152278,
			["ShatteringThrow"] = 64382,
			["IntotheFray"] = 202603,
			["Bolster"] = 280001,
			["HonedReflexes"] = 391271,
			["ImmovableObject"] = 394307,
			["HeroicLeap"] = 6544,
			["EnduranceTraining"] = 382940,
			["Juggernaut"] = 393967,
			["BattleShout"] = 6673,
			["Punish"] = 275334,
			["Execute"] = 163201,
			["DefensiveStance"] = 386208,
			["BloodandThunder"] = 384277,
			["BerserkerRage"] = 18499,
			["ShieldSpecialization"] = 386011,
			["CruelStrikes"] = 392777,
			["BattleStance"] = 386164,
			["ConcussiveBlows"] = 383115,
			["EnduringDefenses"] = 386027,
			["OverwhelmingRage"] = 382767,
			["TitanicThrow"] = 384090,
			["RallyingCry"] = 97462,
			["BattleScarredVeteran"] = 386394,
			["CacophonousRoar"] = 382954,
			["CrushingForce"] = 390642,
			["BoundingStride"] = 202163,
			["FrothingBerserker"] = 392790,
			["WarMachine"] = 316733,
			["IgnorePain"] = 190456,
			["BitterImmunity"] = 383762,
			["Rend"] = 394062,
			["BrutalVitality"] = 384036,
			["DemoralizingShout"] = 1160,
			["Indomitable"] = 202095,
			["PiercingChallenge"] = 382948,
			["StormBolt"] = 107570,
			["ImprovedHeroicThrow"] = 386034,
			["ReinforcedPlates"] = 382939,
			["Menace"] = 275338,
			["ShowofForce"] = 385843,
			["CracklingThunder"] = 203201,
			["ThunderClap"] = 6343,
			["FocusedVigor"] = 384067,
			["Thunderlord"] = 385840,
			["StormofSteel"] = 382953,
			["WreckingThrow"] = 384110,
			["SpellReflection"] = 23920,
			["SonicBoom"] = 390725,
			["ShieldWall"] = 871,
			["Intervene"] = 3411,
			["WildStrikes"] = 382946,
			["ChallengingShout"] = 1161,
			["ViolentOutburst"] = 386477,
			["BoomingVoice"] = 202743,
			["Devastate"] = 20243,
			["SpellBlock"] = 392966,
			["Massacre"] = 281001,
			},
			["Arms"] = {
			["SecondWind"] = 29838,
			["BluntInstruments"] = 383442,
			["Skullsplitter"] = 260643,
			["VictoryRush"] = 34428,
			["CriticalThinking"] = 389306,
			["Massacre"] = 281001,
			["Overpower"] = 7384,
			["BlademastersTorment"] = 390138,
			["Charge"] = 100,
			["LeechingStrikes"] = 382258,
			["Warbreaker"] = 262161,
			["DanceofDeath"] = 390713,
			["Uproar"] = 391572,
			["ArmoredtotheTeeth"] = 384124,
			["TwoHandedWeaponSpecialization"] = 382896,
			["ReapingSwings"] = 383293,
			["ImprovedOverpower"] = 385571,
			["ExecutionersPrecision"] = 386634,
			["Pummel"] = 6552,
			["SuddenDeath"] = 29725,
			["ThunderousWords"] = 384969,
			["MortalStrike"] = 12294,
			["SeismicReverberation"] = 382956,
			["SpellReflection"] = 23920,
			["ChampionsSpear"] = 376079,
			["Taunt"] = 355,
			["ChampionsMight"] = 386285,
			["WarMachine"] = 262231,
			["Sidearm"] = 384404,
			["Hamstring"] = 1715,
			["HeroicThrow"] = 57755,
			["Unhinged"] = 386628,
			["Shockwave"] = 46968,
			["FuriousBlows"] = 390354,
			["SweepingStrikes"] = 260708,
			["Impale"] = 383430,
			["Dreadnaught"] = 262150,
			["TideofBlood"] = 386357,
			["Menace"] = 275338,
			["ThunderousRoar"] = 384318,
			["PainandGain"] = 382549,
			["MercilessBonegrinder"] = 383317,
			["Avatar"] = 107574,
			["MartialProwess"] = 316440,
			["Execute"] = 163201,
			["FervorofBattle"] = 202316,
			["ImprovedExecute"] = 316405,
			["ShieldSlam"] = 23922,
			["EnduranceTraining"] = 382940,
			["DefensiveStance"] = 386208,
			["Tactician"] = 184783,
			["ImpendingVictory"] = 202168,
			["RallyingCry"] = 97462,
			["CollateralDamage"] = 334779,
			["IntimidatingShout"] = 5246,
			["ThunderClap"] = 396719,
			["WreckingThrow"] = 384110,
			["Bloodborne"] = 383287,
			["DoubleTime"] = 103827,
			["Slam"] = 1464,
			["Whirlwind"] = 1680,
			["AngerManagement"] = 152278,
			["ShatteringThrow"] = 64382,
			["BerserkerRage"] = 18499,
			["StormofSwords"] = 385512,
			["ConcussiveBlows"] = 383115,
			["ColossusSmash"] = 167105,
			["Juggernaut"] = 383292,
			["Battlelord"] = 386630,
			["Hurricane"] = 390563,
			["BattleShout"] = 6673,
			["PiercingChallenge"] = 382948,
			["TitanicThrow"] = 384090,
			["ImprovedSweepingStrikes"] = 383155,
			["BloodandThunder"] = 384277,
			["Bladestorm"] = 389774,
			["HeroicLeap"] = 6544,
			["CruelStrikes"] = 392777,
			["BattleStance"] = 386164,
			["WarlordsTorment"] = 390140,
			["OverwhelmingRage"] = 382767,
			["SpitefulSerenity"] = 400314,
			["HonedReflexes"] = 382461,
			["DiebytheSword"] = 118038,
			["ImprovedSlam"] = 400205,
			["CacophonousRoar"] = 382954,
			["RumblingEarth"] = 275339,
			["BoundingStride"] = 202163,
			["FrothingBerserker"] = 392792,
			["StrengthofArms"] = 400803,
			["Cleave"] = 845,
			["BitterImmunity"] = 383762,
			["Rend"] = 772,
			["ExhilaratingBlows"] = 383219,
			["SharpenedBlades"] = 383341,
			["IgnorePain"] = 190456,
			["TestofMight"] = 385008,
			["StormBolt"] = 107570,
			["Fatality"] = 383703,
			["ReinforcedPlates"] = 382939,
			["BerserkerShout"] = 384100,
			["CrushingForce"] = 382764,
			["CracklingThunder"] = 203201,
			["SeasonedSoldier"] = 279423,
			["ShieldBlock"] = 2565,
			["FastFootwork"] = 382260,
			["StormWall"] = 388807,
			["BarbaricTraining"] = 383082,
			["Bloodsurge"] = 384361,
			["SonicBoom"] = 390725,
			["FueledbyViolence"] = 383103,
			["Intervene"] = 3411,
			["WildStrikes"] = 382946,
			["InForTheKill"] = 248621,
			["Bloodletting"] = 383154,
			["DeftExperience"] = 389308,
			["MasteryDeepWounds"] = 262111,
			["PiercingHowl"] = 12323,
			["ValorinVictory"] = 383338,
			},
			["Fury"] = {
			["SecondWind"] = 29838,
			["Charge"] = 100,
			["TitansTorment"] = 390135,
			["SwiftStrikes"] = 383459,
			["Uproar"] = 391572,
			["VictoryRush"] = 34428,
			["CriticalThinking"] = 383297,
			["ImprovedRagingBlow"] = 383854,
			["ArmoredtotheTeeth"] = 384124,
			["RagingBlow"] = 85288,
			["Ravager"] = 228920,
			["FreshMeat"] = 215568,
			["LeechingStrikes"] = 382258,
			["PiercingHowl"] = 12323,
			["Bloodcraze"] = 393950,
			["RagingArmaments"] = 388049,
			["Cruelty"] = 392931,
			["Pummel"] = 6552,
			["ImprovedBloodthirst"] = 383852,
			["ThunderousWords"] = 384969,
			["DeftExperience"] = 383295,
			["SeismicReverberation"] = 382956,
			["Warpaint"] = 208154,
			["SuddenDeath"] = 280721,
			["ImprovedEnrage"] = 383848,
			["SonicBoom"] = 390725,
			["Onslaught"] = 315720,
			["Sidearm"] = 384404,
			["ThunderousRoar"] = 384318,
			["HeroicThrow"] = 57755,
			["SlaughteringStrikes"] = 388004,
			["Bloodthirst"] = 23881,
			["FuriousBlows"] = 390354,
			["ShieldSlam"] = 23922,
			["ChampionsSpear"] = 376079,
			["RumblingEarth"] = 275339,
			["EnragedRegeneration"] = 184364,
			["IntimidatingShout"] = 5246,
			["SpellReflection"] = 23920,
			["SingleMindedFury"] = 81099,
			["ThunderClap"] = 396719,
			["Avatar"] = 107574,
			["Hamstring"] = 1715,
			["CrushingForce"] = 382764,
			["DepthsofInsanity"] = 383922,
			["ImprovedExecute"] = 316402,
			["BarbaricTraining"] = 390674,
			["Recklessness"] = 1719,
			["MasteryUnshackledFury"] = 76856,
			["ChampionsMight"] = 386285,
			["ImpendingVictory"] = 202168,
			["ShatteringThrow"] = 64382,
			["ShieldBlock"] = 2565,
			["UnbridledFerocity"] = 389603,
			["Taunt"] = 355,
			["WreckingThrow"] = 384110,
			["Bloodborne"] = 385703,
			["DoubleTime"] = 103827,
			["Slam"] = 1464,
			["Whirlwind"] = 190411,
			["AngerManagement"] = 152278,
			["Rampage"] = 184367,
			["AshenJuggernaut"] = 392536,
			["Tenderize"] = 388933,
			["HonedReflexes"] = 391270,
			["FrenziedFlurry"] = 383605,
			["TitanicRage"] = 394329,
			["BerserkerShout"] = 384100,
			["DefensiveStance"] = 386208,
			["BattleShout"] = 6673,
			["PiercingChallenge"] = 382948,
			["TitansGrip"] = 46917,
			["RecklessAbandon"] = 396749,
			["BloodandThunder"] = 384277,
			["TitanicThrow"] = 384090,
			["HeroicLeap"] = 6544,
			["DualWield"] = 231842,
			["StormofSwords"] = 388903,
			["ConcussiveBlows"] = 383115,
			["BerserkerRage"] = 18499,
			["Frenzy"] = 335077,
			["WarMachine"] = 346002,
			["RallyingCry"] = 97462,
			["Annihilator"] = 383916,
			["CacophonousRoar"] = 382954,
			["Execute"] = 5308,
			["BoundingStride"] = 202163,
			["OverwhelmingRage"] = 382767,
			["HackandSlash"] = 383877,
			["InvigoratingFury"] = 383468,
			["BitterImmunity"] = 383762,
			["FocusinChaos"] = 383486,
			["FrothingBerserker"] = 215571,
			["Hurricane"] = 390563,
			["BerserkersTorment"] = 390123,
			["ImprovedWhirlwind"] = 12950,
			["StormBolt"] = 107570,
			["CruelStrikes"] = 392777,
			["ReinforcedPlates"] = 382939,
			["EnduranceTraining"] = 391997,
			["WrathandFury"] = 392936,
			["CracklingThunder"] = 203201,
			["ViciousContempt"] = 383885,
			["Menace"] = 275338,
			["FastFootwork"] = 382260,
			["StormofSteel"] = 382953,
			["ColdSteelHotBlood"] = 383959,
			["Enrage"] = 184361,
			["DualWieldSpecialization"] = 382900,
			["PainandGain"] = 382549,
			["Intervene"] = 3411,
			["WildStrikes"] = 382946,
			["Shockwave"] = 46968,
			["Massacre"] = 206315,
			["DancingBlades"] = 391683,
			["MeatCleaver"] = 280392,
			["OdynsFury"] = 385059,
			["BerserkerStance"] = 386196,
			},
		},
		["Paladin"] = {
			["Protection"] = {
			["ImprovedHolyShield"] = 393030,
			["HammerofJustice"] = 853,
			["TurnEvil"] = 10326,
			["Punishment"] = 403530,
			["Riposte"] = 161800,
			["BlessingofProtection"] = 1022,
			["SoaringShield"] = 378457,
			["HammerofWrath"] = 24275,
			["SeasonedWarhorse"] = 376996,
			["FaithsArmor"] = 406101,
			["DivineToll"] = 375576,
			["BlessedHammer"] = 204019,
			["SealofOrder"] = 385129,
			["HammeroftheRighteous"] = 53595,
			["BulwarkofRighteousFury"] = 386653,
			["Redoubt"] = 280373,
			["InmostLight"] = 405757,
			["ImprovedArdentDefender"] = 393114,
			["Cavalier"] = 230332,
			["CleanseToxins"] = 213644,
			["LayonHands"] = 633,
			["CrusaderStrike"] = 35395,
			["SealofMight"] = 385450,
			["UnbreakableSpirit"] = 114154,
			["ResoluteDefender"] = 385422,
			["FlashofLight"] = 19750,
			["GoldenPath"] = 377128,
			["SealofAlacrity"] = 385425,
			["AurasofSwiftVengeance"] = 385639,
			["Consecration"] = 26573,
			["AegisofLight"] = 353367,
			["ShieldoftheRighteous"] = 53600,
			["InnerLight"] = 386568,
			["HolyAegis"] = 385515,
			["ConcentrationAura"] = 317920,
			["Rebuke"] = 96231,
			["DivinePurpose"] = 223817,
			["BlessingofSpellwarding"] = 204018,
			["HandofReckoning"] = 62124,
			["Contemplation"] = 121183,
			["CrusadersJudgment"] = 204023,
			["FerrenMarcussFervor"] = 378762,
			["RighteousProtector"] = 204074,
			["UnboundFreedom"] = 305394,
			["SealofReprisal"] = 377053,
			["RelentlessInquisitor"] = 383388,
			["GiftoftheGoldenValkyr"] = 378279,
			["GrandCrusader"] = 85043,
			["DevotionAura"] = 465,
			["MasteryDivineBulwark"] = 76671,
			["QuickenedInvocation"] = 379391,
			["Judgment"] = 275779,
			["GreaterJudgment"] = 231663,
			["DivineShield"] = 642,
			["FistofJustice"] = 234299,
			["AurasoftheResolute"] = 385633,
			["UthersCounsel"] = 378425,
			["JudgmentofLight"] = 183778,
			["Intercession"] = 391054,
			["BastionofLight"] = 378974,
			["ConsecratedGround"] = 204054,
			["MomentofGlory"] = 327193,
			["StrengthinAdversity"] = 393071,
			["Sanctuary"] = 379021,
			["SacrificeoftheJust"] = 384820,
			["DivineSteed"] = 190784,
			["FocusedEnmity"] = 378845,
			["Sentinel"] = 389539,
			["AvengingWrath"] = 31884,
			["FinalStand"] = 204077,
			["GuardianofAncientKings"] = 86659,
			["ArdentDefender"] = 31850,
			["TouchofLight"] = 385349,
			["BlindingLight"] = 115750,
			["SenseUndead"] = 5502,
			["HandoftheProtector"] = 315924,
			["ImprovedBlessingofProtection"] = 384909,
			["BarricadeofFaith"] = 385726,
			["BlessingofFreedom"] = 1044,
			["SanctifiedPlates"] = 402964,
			["EyeofTyr"] = 387174,
			["SealoftheCrusader"] = 385728,
			["SealofMercy"] = 384897,
			["Incandescence"] = 385464,
			["StrengthofConviction"] = 379008,
			["CrusadersReprieve"] = 403042,
			["FadingLight"] = 405768,
			["Recompense"] = 384914,
			["ConsecrationinFlame"] = 379022,
			["BulwarkofOrder"] = 209389,
			["LightoftheTitans"] = 378405,
			["SealofCharity"] = 384815,
			["OfDuskandDawn"] = 409441,
			["FaithintheLight"] = 379043,
			["TyrsEnforcer"] = 378285,
			["ShiningLight"] = 321136,
			["BlessingofSacrifice"] = 6940,
			["Obduracy"] = 385427,
			["WordofGlory"] = 85673,
			["HolyShield"] = 152261,
			["ZealotsParagon"] = 391142,
			["Afterimage"] = 385414,
			["LightforgedBlessing"] = 406468,
			["AvengersShield"] = 31935,
			["DivineResonance"] = 386738,
			["TirionsDevotion"] = 392928,
			["CrusadersResolve"] = 380188,
			["Justification"] = 377043,
			["InspiringVanguard"] = 393022,
			["Redemption"] = 7328,
			["Repentance"] = 20066,
			},
			["Holy"] = {
			["Daybreak"] = 414170,
			["HammerofJustice"] = 853,
			["BlessingofSummer"] = 388007,
			["TurnEvil"] = 10326,
			["GloriousDawn"] = 414065,
			["Punishment"] = 403530,
			["Redemption"] = 7328,
			["BlessingofProtection"] = 1022,
			["LightforgedBlessing"] = 406468,
			["RisingSunlight"] = 414203,
			["Veneration"] = 392938,
			["ConcentrationAura"] = 317920,
			["HolyInfusion"] = 414214,
			["FaithsArmor"] = 406101,
			["DivineToll"] = 375576,
			["MasteryLightbringer"] = 183997,
			["BeaconofVirtue"] = 200025,
			["Justification"] = 377043,
			["TyrsDeliverance"] = 200652,
			["BlessingofFreedom"] = 1044,
			["LightofDawn"] = 85222,
			["BlessingofSpring"] = 388013,
			["TowerofRadiance"] = 231642,
			["Absolution"] = 212056,
			["ShieldoftheRighteous"] = 415091,
			["Contemplation"] = 121183,
			["DivineFavor"] = 210294,
			["OverflowingLight"] = 414127,
			["CleanseToxins"] = 440013,
			["CommandingLight"] = 387781,
			["LayonHands"] = 633,
			["Reclamation"] = 415364,
			["CrusaderStrike"] = 35395,
			["Illumination"] = 414378,
			["UnbreakableSpirit"] = 114154,
			["HandofDivinity"] = 414273,
			["RighteousJudgment"] = 414113,
			["InfusionofLight"] = 53576,
			["AuraMastery"] = 31821,
			["FlashofLight"] = 19750,
			["GoldenPath"] = 377128,
			["GlisteningRadiance"] = 414139,
			["DivinePurpose"] = 223817,
			["AurasofSwiftVengeance"] = 385639,
			["Consecration"] = 26573,
			["Awakening"] = 414195,
			["UnendingLight"] = 387998,
			["LightsProtection"] = 416799,
			["BreakingDawn"] = 387879,
			["HandofReckoning"] = 62124,
			["HolyAegis"] = 385515,
			["BlessedFocus"] = 414708,
			["BeaconoftheLightbringer"] = 197446,
			["InflorescenceoftheSunwell"] = 392907,
			["ShiningRighteousness"] = 414443,
			["EchoingBlessings"] = 387801,
			["SanctifiedPlates"] = 402964,
			["BarrierofFaith"] = 148039,
			["DivineResonance"] = 386738,
			["LightoftheMartyr"] = 183998,
			["Judgment"] = 275773,
			["Awestruck"] = 417855,
			["SanctifiedWrath"] = 53376,
			["AurasoftheResolute"] = 385633,
			["ImprovedCleanse"] = 393024,
			["FadingLight"] = 405768,
			["CrusadersReprieve"] = 403042,
			["DivineProtection"] = 498,
			["RelentlessInquisitor"] = 383388,
			["GreaterJudgment"] = 231644,
			["SealoftheCrusader"] = 416770,
			["FistofJustice"] = 234299,
			["HolyPrism"] = 114165,
			["DivineShield"] = 642,
			["JudgmentofLight"] = 183778,
			["Intercession"] = 391054,
			["Obduracy"] = 385427,
			["SealofAlacrity"] = 385425,
			["Incandescence"] = 385464,
			["CrusadersMight"] = 196926,
			["SealofOrder"] = 385129,
			["SacrificeoftheJust"] = 384820,
			["DivineSteed"] = 190784,
			["BeaconofFaith"] = 156910,
			["SealofMight"] = 385450,
			["AvengingWrath"] = 31884,
			["SealofMercy"] = 384897,
			["BeaconofLight"] = 53563,
			["ImprovedBlessingofProtection"] = 384909,
			["TouchofLight"] = 385349,
			["SenseUndead"] = 5502,
			["DivineRevelations"] = 387808,
			["HolyLight"] = 82326,
			["ResplendentLight"] = 392902,
			["Recompense"] = 384914,
			["MomentofCompassion"] = 387786,
			["UnwaveringSpirit"] = 392911,
			["Cavalier"] = 230332,
			["LightsHammer"] = 114158,
			["Rebuke"] = 96231,
			["GlimmerofLight"] = 287269,
			["StrengthofConviction"] = 379008,
			["EmpyreanLegacy"] = 387170,
			["VanguardsMomentum"] = 416869,
			["HolyShock"] = 20473,
			["DevotionAura"] = 465,
			["BlindingLight"] = 115750,
			["ProtectionofTyr"] = 200430,
			["QuickenedInvocation"] = 379391,
			["OfDuskandDawn"] = 409439,
			["SavedbytheLight"] = 157047,
			["AvengingCrusader"] = 216331,
			["ImbuedInfusions"] = 392961,
			["BlessingofSacrifice"] = 6940,
			["PoweroftheSilverHand"] = 200474,
			["WordofGlory"] = 85673,
			["SeasonedWarhorse"] = 376996,
			["Repentance"] = 20066,
			["LightsConviction"] = 414073,
			["UntemperedDedication"] = 387814,
			["Afterimage"] = 385414,
			["BoundlessSalvation"] = 392951,
			["TirionsDevotion"] = 414720,
			["MaraadsDyingBreath"] = 388018,
			["HammerofWrath"] = 24275,
			["MercifulAuras"] = 183415,
			["Cleanse"] = 4987,
			["DivineGlimpse"] = 387805,
			},
			["Retribution"] = {
			["BlessedChampion"] = 403010,
			["HeartoftheCrusader"] = 406154,
			["FinalVerdict"] = 383328,
			["Punishment"] = 403530,
			["Penitence"] = 403026,
			["BlessingofProtection"] = 1022,
			["TempestoftheLightbringer"] = 383396,
			["Incandescence"] = 385464,
			["HammerofWrath"] = 24275,
			["SeasonedWarhorse"] = 376996,
			["FaithsArmor"] = 406101,
			["DivineToll"] = 375576,
			["Redemption"] = 7328,
			["JusticarsVengeance"] = 215661,
			["Justification"] = 377043,
			["DivineAuxiliary"] = 406158,
			["BlessingofFreedom"] = 1044,
			["ZealotsFervor"] = 403509,
			["HammerofJustice"] = 853,
			["SealofOrder"] = 385129,
			["ConcentrationAura"] = 317920,
			["Cavalier"] = 230332,
			["ShieldoftheRighteous"] = 53600,
			["SwiftJustice"] = 383228,
			["DivineWrath"] = 406872,
			["CleanseToxins"] = 213644,
			["ExecutionSentence"] = 343527,
			["LayonHands"] = 633,
			["BoundlessJudgment"] = 405278,
			["CrusaderStrike"] = 35395,
			["SealofMight"] = 385450,
			["UnbreakableSpirit"] = 114154,
			["TruthsWake"] = 403696,
			["LightforgedBlessing"] = 403479,
			["Contemplation"] = 121183,
			["Consecration"] = 26573,
			["FlashofLight"] = 19750,
			["GoldenPath"] = 377128,
			["SealofAlacrity"] = 385425,
			["DivineArbiter"] = 404306,
			["AurasofSwiftVengeance"] = 385639,
			["HighlordsJudgment"] = 404512,
			["DevotionAura"] = 465,
			["TemplarStrikes"] = 406646,
			["HandofReckoning"] = 62124,
			["ImprovedCrusaderStrike"] = 383254,
			["BladeofJustice"] = 184575,
			["Crusade"] = 231895,
			["JudgeJuryandExecutioner"] = 405607,
			["Rebuke"] = 96231,
			["JudgmentofLight"] = 183778,
			["Jurisdiction"] = 402971,
			["DivinePurpose"] = 408459,
			["BladesofLight"] = 403664,
			["WakeofAshes"] = 255937,
			["DivineResonance"] = 384027,
			["ImprovedJudgment"] = 405461,
			["DivineStorm"] = 53385,
			["RighteousCause"] = 402912,
			["SanctifiedPlates"] = 402964,
			["DivineProtection"] = 403876,
			["DivineShield"] = 642,
			["GuidedPrayer"] = 404357,
			["FadingLight"] = 405768,
			["QuickenedInvocation"] = 379391,
			["Judgment"] = 20271,
			["GreaterJudgment"] = 231663,
			["CrusadingStrikes"] = 404542,
			["Obduracy"] = 385427,
			["AurasoftheResolute"] = 385633,
			["Adjudication"] = 406157,
			["LightsCelerity"] = 403698,
			["Intercession"] = 391054,
			["HealingHands"] = 326734,
			["Sanctify"] = 382536,
			["SearingLight"] = 404540,
			["ImprovedBlessingofProtection"] = 384909,
			["Recompense"] = 384914,
			["SacrificeoftheJust"] = 384820,
			["DivineSteed"] = 190784,
			["VanguardsMomentum"] = 383314,
			["RushofLight"] = 407067,
			["AvengingWrath"] = 31884,
			["SealofMercy"] = 384897,
			["EmpyreanLegacy"] = 387170,
			["MasteryHandofLight"] = 267316,
			["TouchofLight"] = 385349,
			["StrengthofConviction"] = 379008,
			["SenseUndead"] = 5502,
			["EmpyreanPower"] = 326732,
			["BladeofVengeance"] = 403826,
			["ImprovedBladeofJustice"] = 403745,
			["AegisofProtection"] = 403654,
			["VanguardofJustice"] = 406545,
			["Expurgation"] = 383344,
			["HolyBlade"] = 383342,
			["RetributionAura"] = 183435,
			["DivineHammer"] = 198034,
			["InquisitorsIre"] = 403975,
			["CrusadersReprieve"] = 403042,
			["FinalReckoning"] = 343721,
			["ShieldofVengeance"] = 184662,
			["SealoftheCrusader"] = 385728,
			["BlindingLight"] = 115750,
			["ExecutionersWill"] = 406940,
			["InstrumentofRetribution"] = 404752,
			["OfDuskandDawn"] = 385125,
			["SeethingFlames"] = 405355,
			["ConsecratedGround"] = 204054,
			["FistofJustice"] = 234299,
			["BlessingofSacrifice"] = 6940,
			["ConsecratedBlade"] = 404834,
			["WordofGlory"] = 85673,
			["HolyAegis"] = 385515,
			["ArtofWar"] = 406064,
			["Afterimage"] = 385414,
			["CrusaderAura"] = 32223,
			["LightofJustice"] = 404436,
			["BurningCrusade"] = 405289,
			["TurnEvil"] = 10326,
			["JudgmentofJustice"] = 403495,
			["UnboundFreedom"] = 305394,
			["TemplarsVerdict"] = 85256,
			["VengefulWrath"] = 406835,
			["Repentance"] = 20066,
			},
		},
		["Shaman"] = {
			["Enhancement"] = {
			["ForcefulWinds"] = 262647,
			["Stormflurry"] = 344357,
			["NaturesGuardian"] = 30884,
			["FlameShock"] = 188389,
			["PlanesTraveler"] = 381647,
			["Stormblast"] = 319930,
			["StaticAccumulation"] = 384411,
			["Flurry"] = 382888,
			["WindfuryWeapon"] = 33757,
			["WitchDoctorsAncestry"] = 384447,
			["FeralSpirit"] = 51533,
			["EarthbindTotem"] = 2484,
			["CrashingStorms"] = 334308,
			["GowiththeFlow"] = 381678,
			["FrostShock"] = 196840,
			["StormsWrath"] = 392352,
			["ElementalWeapons"] = 384355,
			["FireNova"] = 333974,
			["EarthShield"] = 974,
			["Ascendance"] = 114051,
			["FeralLunge"] = 196884,
			["VoodooMastery"] = 204268,
			["AlphaWolf"] = 198434,
			["ImprovedMaelstromWeapon"] = 383303,
			["ThorimsInvocation"] = 384444,
			["PrimalMaelstrom"] = 384405,
			["FlametongueWeapon"] = 318038,
			["RagingMaelstrom"] = 384143,
			["CalloftheElements"] = 383011,
			["StoneskinTotem"] = 383017,
			["SpiritWalk"] = 58875,
			["CrashLightning"] = 187874,
			["PrimordialWave"] = 375982,
			["StaticCharge"] = 265046,
			["ConvergingStorms"] = 384363,
			["IceStrike"] = 342240,
			["ChainLightning"] = 188443,
			["NaturesFury"] = 381655,
			["Hex"] = 51514,
			["TempestStrikes"] = 428071,
			["TranquilAirTotem"] = 383019,
			["LavaBurst"] = 51505,
			["SurgingShields"] = 382033,
			["LavaLash"] = 60103,
			["AncestralWolfAffinity"] = 382197,
			["ThunderousPaws"] = 378075,
			["SpiritWolf"] = 260878,
			["DoomWinds"] = 384352,
			["LegacyoftheFrostWitch"] = 384450,
			["Enfeeblement"] = 378079,
			["MoltenAssault"] = 334033,
			["TotemicFocus"] = 382201,
			["FarSight"] = 6196,
			["AstralShift"] = 108271,
			["WindsofAlAkir"] = 382215,
			["Thundershock"] = 378779,
			["Purge"] = 370,
			["MaelstromWeapon"] = 187880,
			["SwirlingMaelstrom"] = 384359,
			["HotHand"] = 201900,
			["Reincarnation"] = 20608,
			["ElementalBlast"] = 117014,
			["PoisonCleansingTotem"] = 383013,
			["WindShear"] = 57994,
			["CapacitorTotem"] = 192058,
			["Thunderstorm"] = 51490,
			["Stormstrike"] = 17364,
			["WindfuryTotem"] = 8512,
			["SplinteredElements"] = 382042,
			["LashingFlames"] = 334046,
			["GreaterPurge"] = 378773,
			["LightningShield"] = 192106,
			["ChainHeal"] = 1064,
			["UnrulyWinds"] = 390288,
			["GracefulSpirit"] = 192088,
			["SpiritwalkersAegis"] = 378077,
			["SpiritwalkersGrace"] = 79206,
			["AshenCatalyst"] = 390370,
			["FocusedInsight"] = 381666,
			["OverflowingMaelstrom"] = 384149,
			["RefreshingWaters"] = 393905,
			["ElementalAssault"] = 210853,
			["Sundering"] = 197214,
			["ElementalSpirits"] = 262624,
			["GhostWolf"] = 2645,
			["CreationCore"] = 383012,
			["GustofWind"] = 192063,
			["EarthgrabTotem"] = 51485,
			["WindRushTotem"] = 192077,
			["BrimmingwithLife"] = 381689,
			["TremorTotem"] = 8143,
			["Hailstorm"] = 334195,
			["WaterWalking"] = 546,
			["SwirlingCurrents"] = 378094,
			["NaturesSwiftness"] = 378081,
			["AstralBulwark"] = 377933,
			["MasteryEnhancedElements"] = 77223,
			["CleanseSpirit"] = 51886,
			["FireandIce"] = 382886,
			["AncestralSpirit"] = 2008,
			["EarthElemental"] = 198103,
			["LightningLasso"] = 305483,
			["HealingStreamTotem"] = 5394,
			["AncestralDefense"] = 382947,
			["AncestralGuidance"] = 108281,
			["ManaSpring"] = 381930,
			["TotemicSurge"] = 381867,
			["TotemicRecall"] = 108285,
			["LightningBolt"] = 188196,
			["Bloodlust"] = 2825,
			["DeeplyRootedElements"] = 378270,
			["TotemicProjection"] = 108287,
			["ElementalWarding"] = 381650,
			["GuardiansCudgel"] = 381819,
			["HealingSurge"] = 8004,
			["AstralRecall"] = 556,
			["PrimalStrike"] = 73899,
			["ElementalOrbit"] = 383010,
			},
			["Restoration"] = {
			["ManaTideTotem"] = 16191,
			["EarthenHarmony"] = 382020,
			["TidalWaves"] = 51564,
			["CurrentControl"] = 404015,
			["EarthShield"] = 974,
			["Downpour"] = 207778,
			["Reincarnation"] = 20608,
			["FlameShock"] = 188389,
			["PlanesTraveler"] = 381647,
			["Ascendance"] = 114052,
			["Wellspring"] = 197995,
			["Flurry"] = 382888,
			["MasteroftheElements"] = 16166,
			["LivingStream"] = 382482,
			["HealingSurge"] = 8004,
			["PurifySpirit"] = 77130,
			["EarthbindTotem"] = 2484,
			["Resurgence"] = 16196,
			["AcidRain"] = 378443,
			["GowiththeFlow"] = 381678,
			["ManaSpring"] = 381930,
			["LightningLasso"] = 305483,
			["Undercurrent"] = 382194,
			["FrostShock"] = 196840,
			["HealingStreamTotem"] = 5394,
			["CloudburstTotem"] = 157153,
			["AncestralProtectionTotem"] = 207399,
			["Tidebringer"] = 236501,
			["HealingWave"] = 77472,
			["Deluge"] = 200076,
			["VoodooMastery"] = 204268,
			["WindShear"] = 57994,
			["Purge"] = 370,
			["EchooftheElements"] = 333919,
			["Hex"] = 51514,
			["FlametongueWeapon"] = 318038,
			["AncestralWolfAffinity"] = 382197,
			["CalloftheElements"] = 383011,
			["TotemicSurge"] = 381867,
			["SpiritWalk"] = 58875,
			["TumblingWaves"] = 382040,
			["UnleashLife"] = 73685,
			["StaticCharge"] = 265046,
			["ImprovedPrimordialWave"] = 382191,
			["PoisonCleansingTotem"] = 383013,
			["ThunderousPaws"] = 378075,
			["HighTide"] = 157154,
			["DeeplyRootedElements"] = 378270,
			["TotemicProjection"] = 108287,
			["TranquilAirTotem"] = 383019,
			["SpiritWolf"] = 260878,
			["PrimordialWave"] = 428332,
			["LavaBurst"] = 51505,
			["FlashFlood"] = 280614,
			["EarthenWallTotem"] = 198838,
			["Riptide"] = 61295,
			["AncestralVision"] = 212048,
			["AncestralGuidance"] = 108281,
			["Enfeeblement"] = 378079,
			["ElementalWarding"] = 381650,
			["ImprovedPurifySpirit"] = 383016,
			["FarSight"] = 6196,
			["AstralShift"] = 108271,
			["WaterShield"] = 52127,
			["Thundershock"] = 378779,
			["ContinuousWaves"] = 382046,
			["AncestralVigor"] = 207401,
			["MaelstromWeapon"] = 187880,
			["SpiritwalkersAegis"] = 378077,
			["EarthlivingWeapon"] = 382021,
			["SpiritwalkersTidalTotem"] = 404522,
			["GreaterPurge"] = 378773,
			["SpiritLinkTotem"] = 98008,
			["CapacitorTotem"] = 192058,
			["Undulation"] = 200071,
			["EarthElemental"] = 198103,
			["AstralBulwark"] = 377933,
			["TideTurner"] = 404019,
			["GhostWolf"] = 2645,
			["WindRushTotem"] = 192077,
			["LightningShield"] = 192106,
			["FlowoftheTides"] = 382039,
			["TremorTotem"] = 8143,
			["AncestralAwakening"] = 382309,
			["CleanseSpirit"] = 440012,
			["ImprovedEarthlivingWeapon"] = 382315,
			["CreationCore"] = 383012,
			["Torrent"] = 200072,
			["Stormkeeper"] = 383009,
			["RefreshingWaters"] = 378211,
			["WavespeakersBlessing"] = 381946,
			["AncestralReach"] = 382732,
			["GracefulSpirit"] = 192088,
			["BrimmingwithLife"] = 381689,
			["PrimalTideCore"] = 382045,
			["NaturesFury"] = 381655,
			["GustofWind"] = 192063,
			["HealingTideTotem"] = 108280,
			["ResonantWaters"] = 404539,
			["WaterTotemMastery"] = 382030,
			["SpiritwalkersGrace"] = 79206,
			["WaterWalking"] = 546,
			["SwirlingCurrents"] = 378094,
			["NaturesSwiftness"] = 378081,
			["Thunderstorm"] = 51490,
			["ChainHeal"] = 1064,
			["WindsofAlAkir"] = 382215,
			["FireandIce"] = 382886,
			["AncestralSpirit"] = 2008,
			["EarthgrabTotem"] = 51485,
			["HealingRain"] = 73920,
			["OverflowingShores"] = 383222,
			["AncestralDefense"] = 382947,
			["MasteryDeepHealing"] = 77226,
			["TotemicFocus"] = 382201,
			["ChainLightning"] = 188443,
			["TotemicRecall"] = 108285,
			["LightningBolt"] = 188196,
			["SurgingShields"] = 382033,
			["StoneskinTotem"] = 383017,
			["PrimalStrike"] = 73899,
			["Bloodlust"] = 2825,
			["GuardiansCudgel"] = 381819,
			["LavaSurge"] = 77756,
			["AstralRecall"] = 556,
			["NaturesGuardian"] = 30884,
			["ElementalOrbit"] = 383010,
			},
			["Elemental"] = {
			["UnrelentingCalamity"] = 382685,
			["MountainsWillFall"] = 381726,
			["Reincarnation"] = 20608,
			["NaturesGuardian"] = 30884,
			["FlamesoftheCauldron"] = 378266,
			["PlanesTraveler"] = 381647,
			["HealingSurge"] = 8004,
			["LavaSurge"] = 77756,
			["Flurry"] = 382888,
			["MasteroftheElements"] = 16166,
			["Ascendance"] = 114050,
			["SwellingMaelstrom"] = 381707,
			["LiquidMagmaTotem"] = 192222,
			["SurgeofPower"] = 262303,
			["LightningLasso"] = 305483,
			["Icefury"] = 210714,
			["GowiththeFlow"] = 381678,
			["FlowofPower"] = 385923,
			["PrimordialWave"] = 375982,
			["EarthShield"] = 974,
			["FrostShock"] = 196840,
			["FluxMelting"] = 381776,
			["Aftershock"] = 273221,
			["HealingStreamTotem"] = 5394,
			["VoodooMastery"] = 204268,
			["FireElemental"] = 198067,
			["PoweroftheMaelstrom"] = 191861,
			["PrimalStrike"] = 73899,
			["Inundate"] = 378776,
			["ManaSpring"] = 381930,
			["MasteryElementalOverload"] = 168534,
			["TotemicSurge"] = 381867,
			["FlametongueWeapon"] = 318038,
			["ElectrifiedShocks"] = 382086,
			["CalloftheElements"] = 383011,
			["StoneskinTotem"] = 383017,
			["SpiritWalk"] = 58875,
			["CallofFire"] = 378255,
			["Earthquake"] = 61882,
			["StaticCharge"] = 265046,
			["OathoftheFarSeer"] = 381785,
			["TotemicFocus"] = 382201,
			["ThunderousPaws"] = 378075,
			["NaturesFury"] = 381655,
			["DeeplyRootedElements"] = 378270,
			["TotemicProjection"] = 108287,
			["TranquilAirTotem"] = 383019,
			["EchoesofGreatSundering"] = 384087,
			["WindShear"] = 57994,
			["LavaBurst"] = 51505,
			["AncestralWolfAffinity"] = 382197,
			["MagmaChamber"] = 381932,
			["SpiritWolf"] = 260878,
			["EchooftheElements"] = 333919,
			["StormElemental"] = 192249,
			["PrimordialFury"] = 378193,
			["PrimordialBond"] = 381764,
			["FireandIce"] = 382886,
			["ElementalEquilibrium"] = 378271,
			["AstralShift"] = 108271,
			["GustofWind"] = 192063,
			["Thundershock"] = 378779,
			["PoisonCleansingTotem"] = 383013,
			["Purge"] = 370,
			["MaelstromWeapon"] = 187880,
			["WindsofAlAkir"] = 382215,
			["EarthShock"] = 8042,
			["ElementalBlast"] = 117014,
			["GreaterPurge"] = 378773,
			["FurtherBeyond"] = 381787,
			["CapacitorTotem"] = 192058,
			["Thunderstorm"] = 51490,
			["EarthElemental"] = 198103,
			["GhostWolf"] = 2645,
			["EyeoftheStorm"] = 381708,
			["CallofThunder"] = 378241,
			["SplinteredElements"] = 382042,
			["LightningShield"] = 192106,
			["TumultuousFissures"] = 381743,
			["SpiritwalkersAegis"] = 378077,
			["WindspeakersLavaResurgence"] = 378268,
			["CleanseSpirit"] = 51886,
			["SpiritwalkersGrace"] = 79206,
			["EchoChamber"] = 382032,
			["FocusedInsight"] = 381666,
			["Stormkeeper"] = 191634,
			["RefreshingWaters"] = 378211,
			["Enfeeblement"] = 378079,
			["RollingMagma"] = 386443,
			["GracefulSpirit"] = 192088,
			["Hex"] = 51514,
			["EarthgrabTotem"] = 51485,
			["EarthbindTotem"] = 2484,
			["WindRushTotem"] = 192077,
			["ElementalWarding"] = 381650,
			["BrimmingwithLife"] = 381689,
			["TremorTotem"] = 8143,
			["AstralBulwark"] = 377933,
			["LightningRod"] = 210689,
			["SwirlingCurrents"] = 378094,
			["NaturesSwiftness"] = 378081,
			["WaterWalking"] = 546,
			["ChainHeal"] = 1064,
			["SkybreakersFieryDemise"] = 378310,
			["ElementalFury"] = 60188,
			["AncestralSpirit"] = 2008,
			["FarSight"] = 6196,
			["AncestralGuidance"] = 108281,
			["ChainLightning"] = 188443,
			["AncestralDefense"] = 382947,
			["PrimordialSurge"] = 386474,
			["SearingFlames"] = 381782,
			["FlashofLightning"] = 381936,
			["TotemicRecall"] = 108285,
			["LightningBolt"] = 188196,
			["SurgingShields"] = 382033,
			["Bloodlust"] = 2825,
			["FlameShock"] = 188389,
			["PrimalElementalist"] = 117013,
			["GuardiansCudgel"] = 381819,
			["ImprovedFlametongueWeapon"] = 382027,
			["AstralRecall"] = 556,
			["CreationCore"] = 383012,
			["ElementalOrbit"] = 383010,
			},
		},
		["Monk"] = {
			["Windwalker"] = {
			["CalmingPresence"] = 388664,
			["GenerousPour"] = 389575,
			["MarkoftheCrane"] = 220357,
			["Skyreach"] = 392991,
			["FortifyingBrew"] = 115203,
			["JadefireHarmony"] = 391412,
			["Ascension"] = 115396,
			["LastEmperorsCapacitor"] = 392989,
			["Transcendence"] = 101643,
			["DrinkingHornCover"] = 391370,
			["JadefireStomp"] = 388193,
			["XuensBond"] = 392986,
			["Serenity"] = 152173,
			["InvokersDelight"] = 388661,
			["StrengthofSpirit"] = 387276,
			["ProfoundRebuttal"] = 392910,
			["TouchoftheTiger"] = 388856,
			["DanceoftheWind"] = 432181,
			["TouchofKarma"] = 122470,
			["ZenPilgrimage"] = 126892,
			["GraceoftheCrane"] = 388811,
			["ResonantFists"] = 389578,
			["SoothingMist"] = 115175,
			["EyeoftheTiger"] = 196607,
			["Paralysis"] = 115078,
			["TigerTailSweep"] = 264348,
			["CraneVortex"] = 388848,
			["TransferthePower"] = 195300,
			["CracklingJadeLightning"] = 117952,
			["Windwalking"] = 157411,
			["Thunderfist"] = 392985,
			["MysticTouch"] = 8647,
			["ImprovedRoll"] = 328669,
			["Attenuation"] = 386941,
			["WideningWhirl"] = 388846,
			["HardenedSoles"] = 391383,
			["VigorousExpulsion"] = 392900,
			["Roll"] = 109132,
			["Afterlife"] = 116092,
			["LegSweep"] = 119381,
			["StormEarthandFire"] = 137639,
			["RushingJadeWind"] = 116847,
			["ExpeditiousFortification"] = 388813,
			["JadeIgnition"] = 392979,
			["ExpelHarm"] = 322101,
			["ImprovedVivify"] = 231602,
			["TeachingsoftheMonastery"] = 116645,
			["ChiWave"] = 115098,
			["TouchofDeath"] = 322109,
			["ChiTorpedo"] = 115008,
			["Resuscitate"] = 115178,
			["WhirlingDragonPunch"] = 152175,
			["XuensBattlegear"] = 392993,
			["Vivify"] = 116670,
			["StrikeoftheWindlord"] = 392983,
			["Detox"] = 218164,
			["DanceofChiJi"] = 325201,
			["OpenPalmStrikes"] = 392970,
			["TigersLust"] = 116841,
			["InvokeXuentheWhiteTiger"] = 123904,
			["ShadowboxingTreads"] = 392982,
			["SpiritualFocus"] = 280197,
			["EmpoweredTigerLightning"] = 323999,
			["FlashingFists"] = 388854,
			["FuryofXuen"] = 396166,
			["FistsofFury"] = 113656,
			["ForbiddenTechnique"] = 393098,
			["DiffuseMagic"] = 122783,
			["GloryoftheDawn"] = 392958,
			["TranscendenceTransfer"] = 119996,
			["SpearHandStrike"] = 116705,
			["Celerity"] = 115173,
			["Skytouch"] = 405044,
			["FlyingSerpentKick"] = 101545,
			["TigerPalm"] = 100780,
			["Provoke"] = 115546,
			["BonedustBrew"] = 386276,
			["ClosetoHeart"] = 389574,
			["RingofPeace"] = 116844,
			["Disable"] = 116095,
			["SongofChiJi"] = 198898,
			["DampenHarm"] = 122278,
			["ImprovedParalysis"] = 344359,
			["ImprovedTouchofDeath"] = 322113,
			["PathofJade"] = 392994,
			["VivaciousVivification"] = 388812,
			["RisingStar"] = 388849,
			["MeridianStrikes"] = 391330,
			["FatalFlyingGuillotine"] = 394923,
			["DustintheWind"] = 394093,
			["FerocityofXuen"] = 388674,
			["IronshellBrew"] = 388814,
			["EscapefromReality"] = 394110,
			["SummonWhiteTigerStatue"] = 388686,
			["MasteryComboStrikes"] = 115636,
			["YulonsGrace"] = 414131,
			["BlackoutKick"] = 100784,
			["RisingSunKick"] = 107428,
			["ChiBurst"] = 123986,
			["SummonJadeSerpentStatue"] = 115313,
			["SaveThemAll"] = 389579,
			["FastFeet"] = 388809,
			["HitCombo"] = 196740,
			["InnerPeace"] = 397768,
			["SummonBlackOxStatue"] = 115315,
			["PowerStrikes"] = 121817,
			["FatalTouch"] = 394123,
			["ElusiveMists"] = 388681,
			["HastyProvocation"] = 328670,
			["BounceBack"] = 389577,
			["SpinningCraneKick"] = 101546,
			["ZenFlight"] = 125883,
			},
			["Brewmaster"] = {
			["CalmingPresence"] = 388664,
			["GracefulExit"] = 387256,
			["GenerousPour"] = 389575,
			["CelestialBrew"] = 322507,
			["MysticTouch"] = 8647,
			["FortifyingBrew"] = 115203,
			["ImprovedParalysis"] = 344359,
			["ElusiveMists"] = 388681,
			["HighTolerance"] = 196737,
			["HealingElixir"] = 122281,
			["KegSmash"] = 121253,
			["IronshellBrew"] = 388814,
			["ZenPilgrimage"] = 126892,
			["CelestialFlames"] = 325177,
			["VivaciousVivification"] = 388812,
			["ResonantFists"] = 389578,
			["Paralysis"] = 115078,
			["QuickSip"] = 388505,
			["CharredPassions"] = 386965,
			["StrengthofSpirit"] = 387276,
			["ProfoundRebuttal"] = 392910,
			["CracklingJadeLightning"] = 117952,
			["Detox"] = 218164,
			["LightBrewing"] = 325093,
			["DanceoftheWind"] = 414132,
			["ImprovedInvokeNiuzaotheBlackOx"] = 322740,
			["CalltoArms"] = 397251,
			["GraceoftheCrane"] = 388811,
			["BobandWeave"] = 280515,
			["SoothingMist"] = 115175,
			["EyeoftheTiger"] = 196607,
			["BountifulBrew"] = 386949,
			["ImprovedCelestialBrew"] = 322510,
			["Roll"] = 109132,
			["InvokeNiuzaotheBlackOx"] = 132578,
			["ImprovedPurifyingBrew"] = 343743,
			["Windwalking"] = 157411,
			["ScaldingBrew"] = 383698,
			["TouchofDeath"] = 322109,
			["ImprovedRoll"] = 328669,
			["Attenuation"] = 386941,
			["FortifyingBrewDetermination"] = 322960,
			["ChiWave"] = 115098,
			["BlackOxBrew"] = 115399,
			["BonedustBrew"] = 386276,
			["MasteryElusiveBrawler"] = 117906,
			["GaiPlinsImperialBrew"] = 383700,
			["PresstheAdvantage"] = 418359,
			["RushingJadeWind"] = 116847,
			["ExpeditiousFortification"] = 388813,
			["DiffuseMagic"] = 122783,
			["ExpelHarm"] = 322101,
			["LegSweep"] = 119381,
			["DragonfireBrew"] = 383994,
			["BlackoutCombo"] = 196736,
			["SpiritoftheOx"] = 400629,
			["ChiTorpedo"] = 115008,
			["Resuscitate"] = 115178,
			["RisingSunKick"] = 107428,
			["Clash"] = 324312,
			["Vivify"] = 116670,
			["SummonWhiteTigerStatue"] = 388686,
			["HitScheme"] = 383695,
			["RingofPeace"] = 116844,
			["TigersLust"] = 116841,
			["StormstoutsLastKeg"] = 383707,
			["FluidityofMotion"] = 387230,
			["WeaponsofOrder"] = 387184,
			["ExplodingKeg"] = 325153,
			["TigerTailSweep"] = 264348,
			["SummonBlackOxStatue"] = 115315,
			["FacePalm"] = 389942,
			["SpearHandStrike"] = 116705,
			["Counterstrike"] = 383785,
			["GiftoftheOx"] = 124502,
			["SpecialDelivery"] = 196730,
			["TranscendenceTransfer"] = 119996,
			["ChiSurge"] = 393400,
			["Celerity"] = 115173,
			["FastFeet"] = 388809,
			["FerocityofXuen"] = 388674,
			["TigerPalm"] = 100780,
			["Provoke"] = 115546,
			["ChiBurst"] = 123986,
			["ClosetoHeart"] = 389574,
			["Anvil&Stave"] = 386937,
			["Disable"] = 116095,
			["SongofChiJi"] = 198898,
			["DampenHarm"] = 122278,
			["Shuffle"] = 322120,
			["VigorousExpulsion"] = 392900,
			["ImprovedTouchofDeath"] = 322113,
			["SummonJadeSerpentStatue"] = 115313,
			["SaveThemAll"] = 389579,
			["PurifyingBrew"] = 119582,
			["StaggeringStrikes"] = 387625,
			["TranquilSpirit"] = 393357,
			["CelestialFortune"] = 216519,
			["ElusiveFootwork"] = 387046,
			["EscapefromReality"] = 394110,
			["TrainingofNiuzao"] = 383714,
			["ShadowboxingTreads"] = 387638,
			["YulonsGrace"] = 414131,
			["BlackoutKick"] = 205523,
			["PretenseofInstability"] = 393516,
			["BrewmastersBalance"] = 245013,
			["FundamentalObservation"] = 387035,
			["SalsalabimsStrength"] = 383697,
			["BreathofFire"] = 115181,
			["ImprovedVivify"] = 231602,
			["ZenMeditation"] = 115176,
			["Stagger"] = 115069,
			["WalkwiththeOx"] = 387219,
			["FatalTouch"] = 394123,
			["Transcendence"] = 101643,
			["HastyProvocation"] = 328670,
			["BounceBack"] = 389577,
			["SpinningCraneKick"] = 322729,
			["ZenFlight"] = 125883,
			},
			["Mistweaver"] = {
			["CalmingPresence"] = 388664,
			["Revival"] = 115310,
			["GenerousPour"] = 389575,
			["ZenPulse"] = 124081,
			["Upwelling"] = 274963,
			["AwakenedJadefire"] = 388779,
			["FortifyingBrew"] = 115203,
			["ImprovedParalysis"] = 344359,
			["RefreshingJadeWind"] = 196725,
			["ElusiveMists"] = 388681,
			["LegacyofWisdom"] = 404408,
			["IronshellBrew"] = 388814,
			["Transcendence"] = 101643,
			["FontofLife"] = 337209,
			["JadefireStomp"] = 388193,
			["InvokeYulontheJadeSerpent"] = 322118,
			["ResonantFists"] = 389578,
			["MistyPeaks"] = 388682,
			["ResplendentMist"] = 388020,
			["InvokersDelight"] = 388661,
			["NourishingChi"] = 387765,
			["Paralysis"] = 115078,
			["EssenceFont"] = 191837,
			["GiftoftheCelestials"] = 388212,
			["StrengthofSpirit"] = 387276,
			["ProfoundRebuttal"] = 392910,
			["EnvelopingMist"] = 124682,
			["CalmingCoalescence"] = 388218,
			["TeachingsoftheMonastery"] = 116645,
			["RenewingMist"] = 115151,
			["ZenPilgrimage"] = 126892,
			["TigerTailSweep"] = 264348,
			["GraceoftheCrane"] = 388811,
			["LegSweep"] = 119381,
			["SoothingMist"] = 115175,
			["Restoral"] = 388615,
			["SummonBlackOxStatue"] = 115315,
			["BurstofLife"] = 399226,
			["Roll"] = 109132,
			["SaveThemAll"] = 389579,
			["TearofMorning"] = 387991,
			["UpliftedSpirits"] = 388551,
			["CloudedFocus"] = 388047,
			["YulonsWhisper"] = 388038,
			["ImprovedRoll"] = 328669,
			["InvigoratingMists"] = 274586,
			["DancingMists"] = 388701,
			["ChiWave"] = 115098,
			["VigorousExpulsion"] = 392900,
			["TeaofPlenty"] = 388517,
			["Reawaken"] = 212051,
			["TigersLust"] = 116841,
			["TouchofDeath"] = 322109,
			["InvokeChiJitheRedCrane"] = 325197,
			["ExpeditiousFortification"] = 388813,
			["ShaohaosLessons"] = 400089,
			["ExpelHarm"] = 322101,
			["ThunderFocusTea"] = 116680,
			["VeilofPride"] = 400053,
			["PeacefulMending"] = 388593,
			["MendingProliferation"] = 388509,
			["ChiTorpedo"] = 115008,
			["Resuscitate"] = 115178,
			["DiffuseMagic"] = 122783,
			["OverflowingMists"] = 388511,
			["SheilunsGift"] = 399491,
			["Lifecycles"] = 197915,
			["DanceofChiJi"] = 438439,
			["EnergizingBrew"] = 422031,
			["AncientConcordance"] = 388740,
			["MistWrap"] = 197900,
			["EchoingReverberation"] = 388604,
			["FastFeet"] = 388809,
			["HealingElixir"] = 122280,
			["ImprovedVivify"] = 231602,
			["Chrysalis"] = 202424,
			["FocusedThunder"] = 197895,
			["EyeoftheTiger"] = 196607,
			["VivaciousVivification"] = 388812,
			["Detox"] = 115450,
			["TeaofSerenity"] = 393460,
			["TranscendenceTransfer"] = 119996,
			["MasteryGustofMists"] = 117907,
			["Celerity"] = 115173,
			["RisingMist"] = 274909,
			["MistsofLife"] = 388548,
			["TigerPalm"] = 100780,
			["Provoke"] = 115546,
			["AccumulatingMist"] = 388564,
			["ClosetoHeart"] = 389574,
			["FerocityofXuen"] = 388674,
			["Disable"] = 116095,
			["SongofChiJi"] = 198898,
			["DampenHarm"] = 122278,
			["JadeBond"] = 388031,
			["Windwalking"] = 157411,
			["ImprovedTouchofDeath"] = 322113,
			["PeerIntoPeace"] = 440008,
			["ManaTea"] = 115294,
			["SecretInfusion"] = 388491,
			["RingofPeace"] = 116844,
			["SpearHandStrike"] = 116705,
			["ChiBurst"] = 123986,
			["Unison"] = 388477,
			["EscapefromReality"] = 394110,
			["SummonWhiteTigerStatue"] = 388686,
			["Vivify"] = 116670,
			["YulonsGrace"] = 414131,
			["BlackoutKick"] = 100784,
			["RisingSunKick"] = 107428,
			["LifeCocoon"] = 116849,
			["AncientTeachings"] = 388023,
			["ImprovedDetox"] = 388874,
			["SummonJadeSerpentStatue"] = 115313,
			["CelestialHarmony"] = 343655,
			["DanceoftheWind"] = 432181,
			["CracklingJadeLightning"] = 117952,
			["RapidDiffusion"] = 388847,
			["FatalTouch"] = 394123,
			["MysticTouch"] = 8647,
			["HastyProvocation"] = 328670,
			["BounceBack"] = 389577,
			["SpinningCraneKick"] = 101546,
			["ZenFlight"] = 125883,
			},
		},
		["Rogue"] = {
			["Outlaw"] = {
			["Ruthlessness"] = 14161,
			["Evasion"] = 5277,
			["KeepItRolling"] = 381989,
			["SuperiorMixture"] = 423701,
			["PickPocket"] = 921,
			["ImprovedMainGauche"] = 382746,
			["Featherfoot"] = 423683,
			["ShadowDance"] = 185313,
			["HeavyHitter"] = 381885,
			["FleetFooted"] = 378813,
			["BlindingPowder"] = 256165,
			["GracefulGuile"] = 423647,
			["EchoingReprimand"] = 385616,
			["Blind"] = 2094,
			["PistolShot"] = 185763,
			["DirtyTricks"] = 108216,
			["VirulentPoisons"] = 381543,
			["Audacity"] = 381845,
			["TakeembySurprise"] = 382742,
			["TripleThreat"] = 381894,
			["ImprovedAdrenalineRush"] = 395422,
			["TightSpender"] = 381621,
			["ThistleTea"] = 381623,
			["Riposte"] = 344363,
			["ColdBlood"] = 382245,
			["CheapShot"] = 1833,
			["GrapplingHook"] = 195457,
			["SwiftSlasher"] = 381988,
			["Nightstalker"] = 14062,
			["BetweentheEyes"] = 315341,
			["ResoundingClarity"] = 381622,
			["BladeFlurry"] = 13877,
			["HitandRun"] = 196922,
			["SafeFall"] = 1860,
			["Ambidexterity"] = 381822,
			["RetractableHook"] = 256188,
			["Kick"] = 1766,
			["SummarilyDispatched"] = 381990,
			["NimbleFingers"] = 378427,
			["Stealth"] = 115191,
			["TricksoftheTrade"] = 57934,
			["ImprovedAmbush"] = 381620,
			["Shiv"] = 5938,
			["Sepsis"] = 385408,
			["BladeRush"] = 271877,
			["AcrobaticStrikes"] = 196924,
			["SliceandDice"] = 315496,
			["SinisterStrike"] = 193315,
			["Sap"] = 6770,
			["ShroudofConcealment"] = 114018,
			["Alacrity"] = 193539,
			["DeftManeuvers"] = 381878,
			["PickLock"] = 1804,
			["KidneyShot"] = 408,
			["SleightofHand"] = 381839,
			["Vanish"] = 1856,
			["StingLikeaBee"] = 131511,
			["Feint"] = 1966,
			["AceUpYourSleeve"] = 381828,
			["SoothingDarkness"] = 393970,
			["ImprovedSprint"] = 231691,
			["Detection"] = 56814,
			["CrimsonVial"] = 185311,
			["DeadlyPrecision"] = 381542,
			["GhostlyStrike"] = 196937,
			["Reverberation"] = 394332,
			["KillingSpree"] = 51690,
			["Crackshot"] = 423703,
			["AdrenalineRush"] = 13750,
			["AtrophicPoison"] = 381637,
			["NumbingPoison"] = 5761,
			["Ambush"] = 8676,
			["Vigor"] = 14983,
			["LeechingPoison"] = 280716,
			["CloakofShadows"] = 31224,
			["Lethality"] = 382238,
			["RestlessBlades"] = 79096,
			["GreenskinsWickers"] = 386823,
			["RushedSetup"] = 378803,
			["FloatLikeaButterfly"] = 354897,
			["Blackjack"] = 379005,
			["MasterPoisoner"] = 378436,
			["DancingSteel"] = 272026,
			["AirborneIrritant"] = 200733,
			["ImprovedWoundPoison"] = 319066,
			["ImprovedBetweentheEyes"] = 235484,
			["Sprint"] = 2983,
			["RolltheBones"] = 315508,
			["UnbreakableStride"] = 400804,
			["Recuperator"] = 378996,
			["DeeperStratagem"] = 193531,
			["IronStomach"] = 193546,
			["CheatDeath"] = 31230,
			["MasteryMainGauche"] = 76806,
			["Eviscerate"] = 196819,
			["Shadowstep"] = 36554,
			["Stillshroud"] = 423662,
			["PrecisionShot"] = 428377,
			["Dispatch"] = 2098,
			["Elusiveness"] = 79008,
			["DeadenedNerves"] = 231719,
			["DeviousStratagem"] = 394321,
			["PreciseCuts"] = 381985,
			["LoadedDice"] = 256170,
			["QuickDraw"] = 196938,
			["CombatPotency"] = 61329,
			["Gouge"] = 1776,
			["FatalFlourish"] = 35551,
			["CombatStamina"] = 381877,
			["Distract"] = 1725,
			["CounttheOdds"] = 381982,
			["Subterfuge"] = 108208,
			["Opportunity"] = 279876,
			["Shadowrunner"] = 378807,
			["ThiefsVersatility"] = 381619,
			["UnderhandedUpperHand"] = 424044,
			["FantheHammer"] = 381846,
			["HiddenOpportunity"] = 383281,
			},
			["Subtlety"] = {
			["Inevitability"] = 382512,
			["Evasion"] = 5277,
			["SuperiorMixture"] = 423701,
			["ImprovedShurikenStorm"] = 319951,
			["Featherfoot"] = 423683,
			["DarkShadow"] = 245687,
			["FleetFooted"] = 378813,
			["PickPocket"] = 921,
			["MasteryExecutioner"] = 76808,
			["DarkBrew"] = 382504,
			["IronStomach"] = 193546,
			["Finality"] = 382525,
			["Subterfuge"] = 108208,
			["Blind"] = 2094,
			["Sprint"] = 2983,
			["Shiv"] = 5938,
			["TerrifyingPace"] = 428387,
			["Weaponmaster"] = 193537,
			["VirulentPoisons"] = 381543,
			["Vigor"] = 14983,
			["Lethality"] = 382238,
			["DeadlyPrecision"] = 381542,
			["Shadowstrike"] = 185438,
			["Veiltouched"] = 382017,
			["Stealth"] = 115191,
			["TightSpender"] = 381621,
			["LeechingPoison"] = 280716,
			["GoremawsBite"] = 426591,
			["BlackPowder"] = 319175,
			["Backstab"] = 53,
			["CheapShot"] = 1833,
			["SafeFall"] = 1860,
			["CrimsonVial"] = 185311,
			["ShurikenToss"] = 114014,
			["CloakofShadows"] = 31224,
			["ResoundingClarity"] = 381622,
			["ShroudofConcealment"] = 114018,
			["Feint"] = 1966,
			["WithoutaTrace"] = 382513,
			["ShadowTechniques"] = 196912,
			["CheatDeath"] = 31230,
			["ShadowBlades"] = 121471,
			["PickLock"] = 1804,
			["CloakedinShadows"] = 382515,
			["TheRotten"] = 382015,
			["Shadowrunner"] = 378807,
			["ImprovedAmbush"] = 381620,
			["EphemeralBond"] = 426563,
			["ShotintheDark"] = 257505,
			["KidneyShot"] = 408,
			["ReplicatingShadows"] = 382506,
			["Flagellation"] = 384631,
			["Kick"] = 1766,
			["Sap"] = 6770,
			["ShadowDance"] = 185313,
			["Alacrity"] = 193539,
			["Distract"] = 1725,
			["DeepeningShadows"] = 185314,
			["Detection"] = 56814,
			["ThistleTea"] = 381623,
			["Vanish"] = 1856,
			["DanseMacabre"] = 382528,
			["SymbolsofDeath"] = 212283,
			["GracefulGuile"] = 423647,
			["Stillshroud"] = 423662,
			["ImprovedSprint"] = 231691,
			["ShurikenTornado"] = 277925,
			["AtrophicPoison"] = 381637,
			["PerforatedVeins"] = 382518,
			["Rupture"] = 1943,
			["Reverberation"] = 394332,
			["RushedSetup"] = 378803,
			["FadetoNothing"] = 382514,
			["Gloomblade"] = 200758,
			["AcrobaticStrikes"] = 196924,
			["NumbingPoison"] = 5761,
			["Ambush"] = 8676,
			["ColdBlood"] = 382245,
			["UnbreakableStride"] = 400804,
			["SecretTechnique"] = 280719,
			["WarningSigns"] = 426555,
			["NimbleFingers"] = 378427,
			["SoothingDarkness"] = 393970,
			["ShadowFocus"] = 108209,
			["Shadowcraft"] = 426594,
			["ImprovedShadowDance"] = 393972,
			["MasterPoisoner"] = 378436,
			["Nightstalker"] = 14062,
			["AirborneIrritant"] = 200733,
			["ImprovedWoundPoison"] = 319066,
			["Elusiveness"] = 79008,
			["NightTerrors"] = 277953,
			["ShurikenStorm"] = 197835,
			["ImprovedBackstab"] = 319949,
			["Recuperator"] = 378996,
			["DeeperStratagem"] = 193531,
			["SwiftDeath"] = 394309,
			["TheFirstDance"] = 382505,
			["ImprovedShadowTechniques"] = 394023,
			["Eviscerate"] = 196819,
			["Shadowstep"] = 36554,
			["InvigoratingShadowdust"] = 382523,
			["QuickDecisions"] = 382503,
			["MasterofShadows"] = 196976,
			["RelentlessStrikes"] = 58423,
			["DeadenedNerves"] = 231719,
			["ExhilaratingExecution"] = 428486,
			["EchoingReprimand"] = 385616,
			["DeeperDaggers"] = 382517,
			["SinisterStrike"] = 1752,
			["SliceandDice"] = 315496,
			["Gouge"] = 1776,
			["FindWeakness"] = 91023,
			["SilentStorm"] = 385722,
			["LingeringShadow"] = 382524,
			["Blackjack"] = 379005,
			["Premeditation"] = 343160,
			["TricksoftheTrade"] = 57934,
			["ShadowedFinishers"] = 382511,
			["ShroudedinDarkness"] = 382507,
			["SecretStratagem"] = 394320,
			["PlannedExecution"] = 382508,
			["Sepsis"] = 385408,
			},
			["Assassination"] = {
			["AtrophicPoison"] = 381637,
			["Evasion"] = 5277,
			["SystemicFailure"] = 381652,
			["SuperiorMixture"] = 423701,
			["PickPocket"] = 921,
			["VenomRush"] = 152152,
			["Featherfoot"] = 423683,
			["ShadowDance"] = 185313,
			["CuttotheChase"] = 51667,
			["FleetFooted"] = 378813,
			["BloodyMess"] = 381626,
			["GracefulGuile"] = 423647,
			["Subterfuge"] = 108208,
			["Blind"] = 2094,
			["LethalDose"] = 381640,
			["VirulentPoisons"] = 381543,
			["Vigor"] = 14983,
			["DragonTemperedBlades"] = 381801,
			["SealFate"] = 14190,
			["FlyingDaggers"] = 381631,
			["Stealth"] = 115191,
			["TightSpender"] = 381621,
			["IronWire"] = 196861,
			["PoisonedKnife"] = 185565,
			["CheapShot"] = 1833,
			["ThrownPrecision"] = 381629,
			["ScentofBlood"] = 381799,
			["ResoundingClarity"] = 381622,
			["MasteryPotentAssassin"] = 76803,
			["Rupture"] = 1943,
			["CausticSpatter"] = 421975,
			["SerratedBoneSpike"] = 385424,
			["CheatDeath"] = 31230,
			["ImprovedGarrote"] = 381632,
			["SinisterStrike"] = 1752,
			["NimbleFingers"] = 378427,
			["TwisttheKnife"] = 381669,
			["Shadowrunner"] = 378807,
			["ImprovedAmbush"] = 381620,
			["AmplifyingPoison"] = 381664,
			["Sepsis"] = 385408,
			["DashingScoundrel"] = 381797,
			["AcrobaticStrikes"] = 196924,
			["SafeFall"] = 1860,
			["IronStomach"] = 193546,
			["Sap"] = 6770,
			["ShroudedSuffocation"] = 385478,
			["Alacrity"] = 193539,
			["ZoldyckRecipe"] = 381798,
			["Eviscerate"] = 196819,
			["ImprovedShiv"] = 319032,
			["ThistleTea"] = 381623,
			["Vanish"] = 1856,
			["Envenom"] = 32645,
			["Garrote"] = 703,
			["Kingsbane"] = 385627,
			["Deathmark"] = 360194,
			["ImprovedSprint"] = 231691,
			["ShroudofConcealment"] = 114018,
			["PickLock"] = 1804,
			["KidneyShot"] = 408,
			["LightweightShiv"] = 394983,
			["UnbreakableStride"] = 400804,
			["FatalConcoction"] = 392384,
			["Kick"] = 1766,
			["Feint"] = 1966,
			["ImprovedPoisons"] = 381624,
			["NumbingPoison"] = 5761,
			["Ambush"] = 8676,
			["Gouge"] = 1776,
			["Mutilate"] = 1329,
			["CloakofShadows"] = 31224,
			["Lethality"] = 382238,
			["CrimsonVial"] = 185311,
			["SoothingDarkness"] = 393970,
			["RushedSetup"] = 378803,
			["Stillshroud"] = 423662,
			["ImprovedWoundPoison"] = 319066,
			["MasterPoisoner"] = 378436,
			["Nightstalker"] = 14062,
			["AirborneIrritant"] = 200733,
			["ColdBlood"] = 382245,
			["IntenttoKill"] = 381630,
			["Sprint"] = 2983,
			["EchoingReprimand"] = 385616,
			["PathofBlood"] = 423054,
			["Recuperator"] = 378996,
			["DeeperStratagem"] = 193531,
			["Blindside"] = 328085,
			["ArterialPrecision"] = 400783,
			["DeadlyPoison"] = 2823,
			["PoisonBomb"] = 255544,
			["Shadowstep"] = 36554,
			["MasterAssassin"] = 255989,
			["SanguineBlades"] = 200806,
			["Doomblade"] = 381673,
			["Elusiveness"] = 79008,
			["DeadenedNerves"] = 231719,
			["IndiscriminateCarnage"] = 381802,
			["ViciousVenoms"] = 381634,
			["TinyToxicBlade"] = 381800,
			["CrimsonTempest"] = 121411,
			["SliceandDice"] = 315496,
			["LeechingPoison"] = 280716,
			["Detection"] = 56814,
			["Reverberation"] = 394332,
			["Distract"] = 1725,
			["Blackjack"] = 379005,
			["SuddenDemise"] = 423136,
			["DeadlyPrecision"] = 381542,
			["InternalBleeding"] = 381627,
			["VenomousWounds"] = 79134,
			["Shiv"] = 5938,
			["FanofKnives"] = 51723,
			["TricksoftheTrade"] = 57934,
			},
		},
		["Mage"] = {
			["Frost"] = {
			["FracturedFrost"] = 378448,
			["RemoveCurse"] = 475,
			["TomeofAntonidas"] = 382490,
			["TomeofRhonin"] = 382493,
			["Polymorph"] = 118,
			["DivertedEnergy"] = 382270,
			["Flurry"] = 44614,
			["IncantationofSwiftness"] = 382293,
			["BrainFreeze"] = 190447,
			["PiercingCold"] = 378919,
			["MassPolymorph"] = 383121,
			["SplinteringRay"] = 418733,
			["ArcaneWarding"] = 383092,
			["Blink"] = 1953,
			["RayofFrost"] = 205021,
			["TimeManipulation"] = 387807,
			["DragonsBreath"] = 31661,
			["IceCold"] = 414658,
			["Spellsteal"] = 30449,
			["GlacialAssault"] = 378947,
			["IceBarrier"] = 11426,
			["Shatter"] = 12982,
			["IceNova"] = 157997,
			["PerpetualWinter"] = 378198,
			["Slow"] = 31589,
			["OverflowingEnergy"] = 390218,
			["SlickIce"] = 382144,
			["MasterofTime"] = 342249,
			["Blizzard"] = 190356,
			["SplinteringCold"] = 379049,
			["FrostNova"] = 122,
			["FireBlast"] = 319836,
			["CryoFreeze"] = 382292,
			["FrigidWinds"] = 235224,
			["ColdestSnap"] = 417493,
			["SlowFall"] = 130,
			["IceFloes"] = 108839,
			["Counterspell"] = 2139,
			["EnergizedBarriers"] = 386828,
			["ConjureRefreshment"] = 190336,
			["LonelyWinter"] = 205024,
			["VolatileDetonation"] = 389627,
			["ArcaneIntellect"] = 1459,
			["QuickWitted"] = 382297,
			["TimeAnomaly"] = 383243,
			["BlastWave"] = 157981,
			["DeepShatter"] = 378749,
			["MassBarrier"] = 414660,
			["FrozenOrb"] = 84714,
			["IceCaller"] = 236662,
			["RingofFrost"] = 113724,
			["FrozenTouch"] = 205030,
			["IceWard"] = 205036,
			["IncantersFlow"] = 1463,
			["Subzero"] = 380154,
			["ColdSnap"] = 235219,
			["Invisibility"] = 66,
			["ShiftingPower"] = 382440,
			["WintersBlessing"] = 417489,
			["TemporalWarp"] = 386539,
			["GreaterInvisibility"] = 110959,
			["EverlastingFrost"] = 385167,
			["FreezingRain"] = 270233,
			["Cryopathy"] = 417491,
			["Frostbite"] = 378756,
			["TemporalVelocity"] = 382826,
			["CometStorm"] = 153595,
			["IceLance"] = 30455,
			["MasteryIcicles"] = 76613,
			["SummonWaterElemental"] = 31687,
			["Reduplication"] = 382569,
			["ImprovedFrostNova"] = 343183,
			["Wintertide"] = 378406,
			["ConeofCold"] = 120,
			["IcyVeins"] = 12472,
			["FreezingWinds"] = 382103,
			["Frostbolt"] = 116,
			["MassInvisibility"] = 414664,
			["GlacialSpike"] = 199786,
			["Snowstorm"] = 381706,
			["TimeWarp"] = 80353,
			["FlashFreeze"] = 379993,
			["MirrorImage"] = 55342,
			["FingersofFrost"] = 112965,
			["RigidIce"] = 382481,
			["AlterTime"] = 342245,
			["Hailstones"] = 381244,
			["SplittingIce"] = 56377,
			["ThermalVoid"] = 155149,
			["AccumulativeShielding"] = 382800,
			["ChainReaction"] = 278309,
			["IceBlock"] = 45438,
			["TempestBarrier"] = 382289,
			["ColdFront"] = 382110,
			["ArcaneExplosion"] = 1449,
			["BoneChilling"] = 205027,
			["Reabsorption"] = 382820,
			["FreezingCold"] = 386763,
			["Shimmer"] = 212653,
			["MassSlow"] = 391102,
			["FlowofTime"] = 382268,
			["WintersProtection"] = 382424,
			["Displacement"] = 389713,
			},
			["Fire"] = {
			["RemoveCurse"] = 475,
			["TomeofAntonidas"] = 382490,
			["TomeofRhonin"] = 382493,
			["CriticalMass"] = 117216,
			["Polymorph"] = 118,
			["DivertedEnergy"] = 382270,
			["IncantationofSwiftness"] = 382293,
			["DeepImpact"] = 416719,
			["FeeltheBurn"] = 383391,
			["RigidIce"] = 382481,
			["TempestBarrier"] = 382289,
			["ArcaneWarding"] = 383092,
			["Blink"] = 1953,
			["TimeManipulation"] = 387807,
			["DragonsBreath"] = 31661,
			["AlexstraszasFury"] = 235870,
			["IceCold"] = 414659,
			["Fireball"] = 133,
			["Spellsteal"] = 30449,
			["ImprovedScorch"] = 383604,
			["Conflagration"] = 205023,
			["Pyromaniac"] = 205020,
			["FueltheFire"] = 416094,
			["MirrorImage"] = 55342,
			["Meteor"] = 153561,
			["SunKingsBlessing"] = 383886,
			["ConjureRefreshment"] = 190336,
			["Slow"] = 31589,
			["OverflowingEnergy"] = 390218,
			["HotStreak"] = 195283,
			["ImprovedCombustion"] = 383967,
			["MasterofTime"] = 342249,
			["Invisibility"] = 66,
			["Pyrotechnics"] = 157642,
			["SearingTouch"] = 269644,
			["FlowofTime"] = 382268,
			["FireBlast"] = 108853,
			["CryoFreeze"] = 382292,
			["FlamePatch"] = 205037,
			["IceBlock"] = 45438,
			["SlowFall"] = 130,
			["IceFloes"] = 108839,
			["TemperedFlames"] = 383659,
			["EnergizedBarriers"] = 386828,
			["FrostNova"] = 122,
			["Cauterize"] = 86949,
			["VolatileDetonation"] = 389627,
			["Kindling"] = 155148,
			["Counterspell"] = 2139,
			["TimeAnomaly"] = 383243,
			["ArcaneIntellect"] = 1459,
			["ArcaneExplosion"] = 1449,
			["MassBarrier"] = 414660,
			["Pyroblast"] = 11366,
			["QuickWitted"] = 382297,
			["BlazingBarrier"] = 235313,
			["Hyperthermia"] = 383860,
			["IceWard"] = 205036,
			["IncantersFlow"] = 1463,
			["MasterofFlame"] = 384174,
			["Firemind"] = 383499,
			["Inflame"] = 417467,
			["ShiftingPower"] = 382440,
			["Convection"] = 416715,
			["TemporalWarp"] = 386539,
			["Combustion"] = 190319,
			["FromtheAshes"] = 342344,
			["GreaterInvisibility"] = 110959,
			["TemporalVelocity"] = 382826,
			["SurgingBlaze"] = 343230,
			["CalloftheSunKing"] = 343222,
			["PhoenixFlames"] = 257541,
			["Firefall"] = 384033,
			["Scorch"] = 2948,
			["FeveredIncantation"] = 383810,
			["Reduplication"] = 382569,
			["ImprovedFrostNova"] = 343183,
			["IntensifyingFlame"] = 416714,
			["ConeofCold"] = 120,
			["Firestarter"] = 205026,
			["MasteryIgnite"] = 12846,
			["Frostbolt"] = 116,
			["MassInvisibility"] = 414664,
			["Wildfire"] = 383489,
			["FieryRush"] = 383634,
			["TimeWarp"] = 80353,
			["UnleashedInferno"] = 416506,
			["PhoenixReborn"] = 383476,
			["FlameAccelerant"] = 203275,
			["Flamestrike"] = 2120,
			["AlterTime"] = 342245,
			["AccumulativeShielding"] = 382800,
			["FrigidWinds"] = 235224,
			["RingofFrost"] = 113724,
			["FerventFlickering"] = 387044,
			["LivingBomb"] = 44457,
			["MassPolymorph"] = 383121,
			["FlameOn"] = 205029,
			["IncendiaryEruptions"] = 383665,
			["BlastWave"] = 157981,
			["IceNova"] = 157997,
			["ControlledDestruction"] = 383669,
			["FreezingCold"] = 386763,
			["Shimmer"] = 212653,
			["MassSlow"] = 391102,
			["Reabsorption"] = 382820,
			["WintersProtection"] = 382424,
			["Displacement"] = 389713,
			},
			["Arcane"] = {
			["ConjureManaGem"] = 759,
			["RemoveCurse"] = 475,
			["TomeofAntonidas"] = 382490,
			["TomeofRhonin"] = 382493,
			["ArcaneFamiliar"] = 205022,
			["Polymorph"] = 118,
			["DivertedEnergy"] = 382270,
			["OrbBarrage"] = 384858,
			["RuleofThrees"] = 264354,
			["IncantationofSwiftness"] = 382293,
			["ArcaneBlast"] = 30451,
			["ProdigiousSavant"] = 384612,
			["RigidIce"] = 382481,
			["TempestBarrier"] = 382289,
			["ArcaneWarding"] = 383092,
			["NetherTempest"] = 114923,
			["TimeManipulation"] = 387807,
			["DragonsBreath"] = 31661,
			["IceCold"] = 414659,
			["Spellsteal"] = 30449,
			["ChronoShift"] = 235711,
			["ChargedOrb"] = 384651,
			["IceNova"] = 157997,
			["Concentration"] = 384374,
			["ArcaneHarmony"] = 384452,
			["Slow"] = 31589,
			["OverflowingEnergy"] = 390218,
			["ArcaneMissiles"] = 5143,
			["IlluminatedThoughts"] = 384060,
			["MasterofTime"] = 342249,
			["Enlightened"] = 321387,
			["Supernova"] = 157980,
			["ArcaneSurge"] = 365350,
			["FireBlast"] = 319836,
			["CryoFreeze"] = 382292,
			["FrigidWinds"] = 235224,
			["IceBlock"] = 45438,
			["NetherPrecision"] = 383782,
			["IceFloes"] = 108839,
			["ConjureRefreshment"] = 190336,
			["EnergizedBarriers"] = 386828,
			["Evocation"] = 12051,
			["Amplification"] = 236628,
			["VolatileDetonation"] = 389627,
			["SlowFall"] = 130,
			["Invisibility"] = 66,
			["TimeAnomaly"] = 383243,
			["ArcingCleave"] = 231564,
			["Reverberate"] = 281482,
			["FrostNova"] = 122,
			["ArcaneBarrage"] = 44425,
			["QuickWitted"] = 382297,
			["RingofFrost"] = 113724,
			["CascadingPower"] = 384276,
			["IceWard"] = 205036,
			["IncantersFlow"] = 1463,
			["Counterspell"] = 2139,
			["MassBarrier"] = 414660,
			["ArcaneBombardment"] = 384581,
			["ShiftingPower"] = 382440,
			["Clearcasting"] = 79684,
			["TemporalWarp"] = 386539,
			["ArcaneIntellect"] = 1459,
			["ArcaneExplosion"] = 1449,
			["GreaterInvisibility"] = 110959,
			["TemporalVelocity"] = 382826,
			["PrismaticBarrier"] = 235450,
			["MasterySavant"] = 190740,
			["ArcaneOrb"] = 153626,
			["RadiantSpark"] = 376103,
			["ConcentratedPower"] = 414379,
			["ImprovedArcaneMissiles"] = 383661,
			["Reduplication"] = 382569,
			["ImprovedFrostNova"] = 343183,
			["TouchoftheMagi"] = 321507,
			["ConeofCold"] = 120,
			["ImprovedPrismaticBarrier"] = 321745,
			["CracklingEnergy"] = 321752,
			["Frostbolt"] = 116,
			["MassInvisibility"] = 414664,
			["Slipstream"] = 236457,
			["Impetus"] = 383676,
			["TimeWarp"] = 80353,
			["ImprovedClearcasting"] = 321420,
			["HarmonicEcho"] = 384683,
			["AccumulativeShielding"] = 382800,
			["ManaAdept"] = 321526,
			["AlterTime"] = 342245,
			["Resonance"] = 205028,
			["PresenceofMind"] = 205025,
			["SiphonStorm"] = 384187,
			["ArcaneTempo"] = 383980,
			["Foresight"] = 384861,
			["MassPolymorph"] = 383121,
			["Blink"] = 1953,
			["WintersProtection"] = 382424,
			["BlastWave"] = 157981,
			["Reabsorption"] = 382820,
			["MirrorImage"] = 55342,
			["FreezingCold"] = 386763,
			["Shimmer"] = 212653,
			["MassSlow"] = 391102,
			["ArcaneEcho"] = 342231,
			["FlowofTime"] = 382268,
			["Displacement"] = 389713,
			},
		},
		["Demon Hunter"] = {
			["Havoc"] = {
			["Inertia"] = 427640,
			["ElysianDecree"] = 390163,
			["CriticalChaos"] = 320413,
			["FelBarrage"] = 258925,
			["EssenceBreak"] = 258860,
			["PitchBlack"] = 389783,
			["TrailofRuin"] = 258881,
			["BurningWound"] = 391189,
			["Pursuit"] = 320654,
			["LooksCanKill"] = 320415,
			["UnboundChaos"] = 347461,
			["GrowingInferno"] = 390158,
			["ShatteredSouls"] = 178940,
			["FelEruption"] = 211881,
			["SigilofFlame"] = 204596,
			["DemonHide"] = 428241,
			["ChaosStrike"] = 162794,
			["Blur"] = 198589,
			["MortalDance"] = 328725,
			["AuraofPain"] = 207347,
			["DemonBlades"] = 203555,
			["DemonicAppetite"] = 206478,
			["DemonMuzzle"] = 388111,
			["Netherwalk"] = 196555,
			["ErraticFelheart"] = 391397,
			["BouncingGlaives"] = 320386,
			["Darkness"] = 196718,
			["AcceleratedBlade"] = 391275,
			["ImprovedDisrupt"] = 320361,
			["ChaosBrand"] = 255260,
			["InnerDemon"] = 389693,
			["SigilofMisery"] = 207684,
			["SwallowedAnger"] = 320313,
			["SpectralSight"] = 188501,
			["RestlessHunter"] = 390142,
			["Disrupt"] = 183752,
			["BurningHatred"] = 320374,
			["ShatteredRestoration"] = 389824,
			["SerratedGlaive"] = 390154,
			["Demonic"] = 213410,
			["ConsumeMagic"] = 278326,
			["AnyMeansNecessary"] = 388114,
			["BlindFury"] = 203550,
			["VengefulRetreat"] = 198793,
			["SoulSigils"] = 395446,
			["ImmolationAura"] = 258920,
			["ChampionoftheGlaive"] = 429211,
			["SoulRending"] = 204909,
			["ThrowGlaive"] = 185123,
			["FirstBlood"] = 206416,
			["LivebytheGlaive"] = 428607,
			["UnrestrainedFury"] = 320770,
			["DancingwithFate"] = 389978,
			["WilloftheIllidari"] = 389695,
			["FuriousGaze"] = 343311,
			["FelfireHaste"] = 389846,
			["PreciseSigils"] = 389799,
			["FelRush"] = 195072,
			["IsolatedPrey"] = 388113,
			["CycleofHatred"] = 258887,
			["InsatiableHunger"] = 258876,
			["RelentlessOnslaught"] = 389977,
			["EyeBeam"] = 198013,
			["Ragefire"] = 388107,
			["TacticalRetreat"] = 389688,
			["BladeDance"] = 188499,
			["AldrachiDesign"] = 391409,
			["LongNight"] = 389781,
			["IllidariKnowledge"] = 389696,
			["ShatteredDestiny"] = 388116,
			["Glide"] = 131347,
			["Soulscar"] = 388106,
			["QuickenedSigils"] = 209281,
			["ChaosTheory"] = 389687,
			["RushofChaos"] = 320421,
			["Momentum"] = 206476,
			["ImprovedChaosStrike"] = 343206,
			["AFireInside"] = 427775,
			["ChaoticDisposition"] = 428492,
			["TheHunt"] = 370965,
			["LostinDarkness"] = 389849,
			["DisruptingFury"] = 183782,
			["MasteryDemonicPresence"] = 185164,
			["DemonicWards"] = 278386,
			["KnowYourEnemy"] = 388118,
			["ChaoticTransformation"] = 388112,
			["MasteroftheGlaive"] = 389763,
			["DashofChaos"] = 427794,
			["Metamorphosis"] = 191427,
			["CollectiveAnguish"] = 390152,
			["ChaosFragments"] = 320412,
			["FuriousThrows"] = 393029,
			["DesperateInstincts"] = 205411,
			["FlamesofFury"] = 389694,
			["Imprison"] = 217832,
			["ImprovedSigilofMisery"] = 320418,
			["GlaiveTempest"] = 342817,
			["ChaosNova"] = 179057,
			["InternalStruggle"] = 393822,
			["ScarsofSuffering"] = 428232,
			["Felblade"] = 232893,
			["InfernalArmor"] = 320331,
			["ImprovedFelRush"] = 343017,
			["VengefulBonds"] = 320635,
			["Initiative"] = 388108,
			["DemonsBite"] = 344859,
			["BlazingPath"] = 320416,
			["Torment"] = 185245,
			["CharredWarblades"] = 213010,
			["DoubleJump"] = 196055,
			["DeflectingDance"] = 427776,
			},
			["Vengeance"] = {
			["SigilofFlame"] = 204596,
			["ElysianDecree"] = 390163,
			["InfernalStrike"] = 189110,
			["Torment"] = 185245,
			["IlluminatedSigils"] = 428557,
			["Fracture"] = 263642,
			["CharredWarblades"] = 213010,
			["Soulcrush"] = 389985,
			["FieryBrand"] = 204021,
			["Pursuit"] = 320654,
			["ChainsofAnger"] = 389715,
			["SoulCarver"] = 207407,
			["VolatileFlameblood"] = 390808,
			["FieryDemise"] = 389220,
			["FelDevastation"] = 212084,
			["StoketheFlames"] = 393827,
			["ChaosStrike"] = 344862,
			["CycleofBinding"] = 389718,
			["AuraofPain"] = 207347,
			["DemonsBite"] = 344859,
			["DemonMuzzle"] = 388111,
			["Retaliation"] = 389729,
			["CharredFlesh"] = 336639,
			["ImprovedDisrupt"] = 320361,
			["BouncingGlaives"] = 320386,
			["SigilofChains"] = 202138,
			["ShearFury"] = 389997,
			["LastResort"] = 209258,
			["AscendingFlame"] = 428603,
			["CalcifiedSpikes"] = 389720,
			["SwallowedAnger"] = 320313,
			["SpectralSight"] = 188501,
			["ChaosBrand"] = 255260,
			["Disrupt"] = 183752,
			["DarkglareBoon"] = 389708,
			["RuinousBulwark"] = 326853,
			["VengefulBonds"] = 320635,
			["Painbringer"] = 207387,
			["ConsumeMagic"] = 278326,
			["FeedtheDemon"] = 218612,
			["Soulmonger"] = 389711,
			["PreciseSigils"] = 389799,
			["SoulSigils"] = 395446,
			["ImmolationAura"] = 258920,
			["ChampionoftheGlaive"] = 429211,
			["SoulRending"] = 204909,
			["ThrowGlaive"] = 204157,
			["DisruptingFury"] = 183782,
			["LivebytheGlaive"] = 428607,
			["DemonSpikes"] = 203720,
			["FelfireHaste"] = 389846,
			["InternalStruggle"] = 393822,
			["VengefulRetreat"] = 198793,
			["SoulBarrier"] = 263648,
			["FeastofSouls"] = 207697,
			["FelRush"] = 344865,
			["UnrestrainedFury"] = 320770,
			["Vulnerability"] = 389976,
			["FlamesofFury"] = 389694,
			["SoulFurnace"] = 391165,
			["BurningAlive"] = 207739,
			["TheHunt"] = 370965,
			["FocusedCleave"] = 343207,
			["AldrachiDesign"] = 391409,
			["ExtendedSpikes"] = 389721,
			["DeflectingSpikes"] = 321028,
			["IllidariKnowledge"] = 389696,
			["BulkExtraction"] = 320341,
			["RoaringFire"] = 391178,
			["AgonizingFlames"] = 207548,
			["QuickenedSigils"] = 209281,
			["VoidReaver"] = 268175,
			["RushofChaos"] = 320421,
			["FelFlameFortification"] = 389705,
			["Darkness"] = 196718,
			["PitchBlack"] = 389783,
			["Fallout"] = 227174,
			["Frailty"] = 389958,
			["LostinDarkness"] = 389849,
			["ErraticFelheart"] = 391397,
			["LongNight"] = 389781,
			["DemonicWards"] = 203513,
			["SpiritBomb"] = 247454,
			["WilloftheIllidari"] = 389695,
			["MasteroftheGlaive"] = 389763,
			["SoulCleave"] = 228477,
			["Metamorphosis"] = 187827,
			["CollectiveAnguish"] = 390152,
			["ChaosFragments"] = 320412,
			["Glide"] = 131347,
			["SigilofMisery"] = 207684,
			["BurningBlood"] = 390213,
			["Imprison"] = 217832,
			["ImprovedSigilofMisery"] = 320418,
			["RevelinPain"] = 343014,
			["ChaosNova"] = 179057,
			["Demonic"] = 213410,
			["ShatteredRestoration"] = 389824,
			["Felblade"] = 232893,
			["InfernalArmor"] = 320331,
			["MasteryFelBlood"] = 203747,
			["MeteoricStrikes"] = 389724,
			["DowninFlames"] = 389732,
			["ThickSkin"] = 320380,
			["BlazingPath"] = 320416,
			["SigilofSilence"] = 202137,
			["PerfectlyBalancedGlaive"] = 320387,
			["DoubleJump"] = 196055,
			["ShatteredSouls"] = 204254,
			},
		},
		["Warlock"] = {
			["Demonology"] = {
			["Soulburn"] = 385899,
			["SargereiTechnique"] = 405955,
			["Shadowflame"] = 384069,
			["UnendingBreath"] = 5697,
			["GrimoireofSynergy"] = 171975,
			["Demoniac"] = 426115,
			["SoulLeech"] = 108370,
			["ImpGangBoss"] = 387445,
			["Horrify"] = 56244,
			["CarnivorousStalkers"] = 386194,
			["HeavyHanded"] = 416183,
			["RitualofDoom"] = 342601,
			["WrathfulMinion"] = 386864,
			["ReignofTyranny"] = 427684,
			["FelandSteel"] = 386200,
			["ProfaneBargain"] = 389576,
			["SubjugateDemon"] = 1098,
			["FelArmor"] = 386124,
			["AccruedVitality"] = 386613,
			["AntoranArmaments"] = 387494,
			["Darkfury"] = 264874,
			["DemonicResilience"] = 389590,
			["FelInvocation"] = 428351,
			["GrandWarlocksDesign"] = 387084,
			["SoulShards"] = 246985,
			["AmplifyCurse"] = 328774,
			["CursesofEnfeeblement"] = 386105,
			["ImmutableHatred"] = 405670,
			["Shadowfury"] = 30283,
			["DemonSkin"] = 219272,
			["InnerDemons"] = 267216,
			["NerzhulsVolition"] = 387526,
			["KazaaksFinalCurse"] = 387483,
			["DemonicEmbrace"] = 288843,
			["DarkAccord"] = 386659,
			["DreadCalling"] = 387391,
			["HowlofTerror"] = 5484,
			["Fear"] = 5782,
			["Demonbolt"] = 264178,
			["AbyssWalker"] = 389609,
			["FelDomination"] = 333889,
			["SpitefulReconstitution"] = 428394,
			["CurseofWeakness"] = 702,
			["ShadowsBite"] = 387322,
			["FelSynergy"] = 389367,
			["DemonicInspiration"] = 386858,
			["BurningRush"] = 111400,
			["Imperator"] = 416230,
			["PowerSiphon"] = 264130,
			["DemonicCircle"] = 48018,
			["TheExpendables"] = 387600,
			["MaleficImpact"] = 416341,
			["ShadowBolt"] = 686,
			["CallDreadstalkers"] = 104316,
			["Guillotine"] = 386833,
			["ShadowInvocation"] = 422054,
			["EyeofKilrogg"] = 126,
			["DrainLife"] = 234153,
			["DemonicCore"] = 267102,
			["Implosion"] = 196277,
			["InfernalCommand"] = 387549,
			["SweetSouls"] = 386620,
			["FelPact"] = 386113,
			["IchorofDevils"] = 386664,
			["SoulboundTyrant"] = 334585,
			["Lifeblood"] = 386646,
			["StolenPower"] = 387602,
			["InquisitorsGaze"] = 386344,
			["CreateSoulwell"] = 29893,
			["Banish"] = 710,
			["FiendishStride"] = 386110,
			["Cavitation"] = 416154,
			["FelSunder"] = 387399,
			["CurseofExhaustion"] = 334275,
			["Corruption"] = 172,
			["SocretharsGuile"] = 405936,
			["MasteryMasterDemonologist"] = 77219,
			["Nightmare"] = 386648,
			["TeachingsoftheSatyr"] = 387972,
			["DemonicCalling"] = 205145,
			["TheHoundmastersStratagem"] = 267170,
			["DemonicFortitude"] = 386617,
			["DarkPact"] = 108416,
			["TeachingsoftheBlackHarvest"] = 385881,
			["SpellLock"] = 119910,
			["CurseofTongues"] = 1714,
			["Dreadlash"] = 264078,
			["DemonicCircleTeleport"] = 48020,
			["HandofGuldan"] = 105174,
			["SoulStrike"] = 428344,
			["SacrificedSouls"] = 267214,
			["SoulLink"] = 108415,
			["GrimoireFelguard"] = 111898,
			["DemonicKnowledge"] = 386185,
			["ResoluteBarrier"] = 389359,
			["TheCodexofXerrath"] = 101508,
			["GrimFeast"] = 386689,
			["NetherPortal"] = 267217,
			["AnnihilanTraining"] = 386174,
			["GuldansAmbition"] = 387578,
			["Doom"] = 603,
			["SummonDemonicTyrant"] = 265187,
			["CreateHealthstone"] = 6201,
			["DemonicGateway"] = 111771,
			["FrequentDonor"] = 386686,
			["StrengthofWill"] = 317138,
			["HealthFunnel"] = 755,
			["UmbralBlaze"] = 405798,
			["DemonicStrength"] = 267171,
			["RitualofSummoning"] = 698,
			["PactoftheImpMother"] = 387541,
			["SummonVilefiend"] = 264119,
			["SoulConduit"] = 215941,
			["Soulstone"] = 20707,
			["SummonSoulkeeper"] = 386256,
			["UnendingResolve"] = 104773,
			["CommandDemon"] = 119898,
			["BilescourgeBombers"] = 267211,
			["MortalCoil"] = 6789,
			},
			["Destruction"] = {
			["Soulburn"] = 385899,
			["SargereiTechnique"] = 405955,
			["Shadowflame"] = 384069,
			["UnendingBreath"] = 5697,
			["GrimoireofSynergy"] = 171975,
			["Decimation"] = 387176,
			["Backdraft"] = 196406,
			["SoulLeech"] = 108370,
			["Horrify"] = 56244,
			["Inferno"] = 270545,
			["GrimoireofSacrifice"] = 108503,
			["SummonInfernal"] = 1122,
			["ScaldingFlames"] = 388832,
			["WrathfulMinion"] = 386864,
			["ProfaneBargain"] = 389576,
			["SubjugateDemon"] = 1098,
			["Shadowburn"] = 17877,
			["AccruedVitality"] = 386613,
			["Immolate"] = 348,
			["TheCodexofXerrath"] = 101508,
			["DemonicResilience"] = 389590,
			["MortalCoil"] = 6789,
			["GrandWarlocksDesign"] = 387084,
			["AshenRemains"] = 387252,
			["AmplifyCurse"] = 328774,
			["CursesofEnfeeblement"] = 386105,
			["ImprovedConflagrate"] = 231793,
			["SoulShards"] = 246985,
			["DemonSkin"] = 219272,
			["RollingHavoc"] = 387569,
			["RainofChaos"] = 266086,
			["MasteryChaoticEnergies"] = 77220,
			["DemonicEmbrace"] = 288843,
			["DarkAccord"] = 386659,
			["SoulLink"] = 108415,
			["HowlofTerror"] = 5484,
			["EyeofKilrogg"] = 126,
			["SoulConduit"] = 215941,
			["DiabolicEmbers"] = 387173,
			["FelDomination"] = 333889,
			["AbyssWalker"] = 389609,
			["FrequentDonor"] = 386686,
			["PowerOverwhelming"] = 387279,
			["FelSynergy"] = 389367,
			["DemonicInspiration"] = 386858,
			["ReverseEntropy"] = 205148,
			["Chaosbringer"] = 422057,
			["DemonicCircle"] = 48018,
			["Havoc"] = 80240,
			["DemonicGateway"] = 111771,
			["StrengthofWill"] = 317138,
			["ShadowBolt"] = 686,
			["ChannelDemonfire"] = 196447,
			["Shadowfury"] = 30283,
			["IchorofDevils"] = 386664,
			["ChaosBolt"] = 116858,
			["DrainLife"] = 234153,
			["InfernalBrand"] = 387475,
			["Cataclysm"] = 152108,
			["RagingDemonfire"] = 387166,
			["SweetSouls"] = 386620,
			["FelPact"] = 386113,
			["Pyrogenics"] = 387095,
			["ResoluteBarrier"] = 389359,
			["TeachingsoftheBlackHarvest"] = 385881,
			["ExplosivePotential"] = 388827,
			["InquisitorsGaze"] = 386344,
			["CreateSoulwell"] = 29893,
			["Pandemonium"] = 387509,
			["FiendishStride"] = 386110,
			["RitualofRuin"] = 387156,
			["RitualofDoom"] = 342601,
			["CurseofExhaustion"] = 334275,
			["CurseofTongues"] = 1714,
			["SocretharsGuile"] = 405936,
			["Banish"] = 710,
			["CrashingChaos"] = 417234,
			["TeachingsoftheSatyr"] = 387972,
			["Flashpoint"] = 387259,
			["DarkPact"] = 108416,
			["DemonicFortitude"] = 386617,
			["MasterRitualist"] = 387165,
			["Nightmare"] = 386648,
			["SpellLock"] = 119910,
			["CryHavoc"] = 387522,
			["BurningRush"] = 111400,
			["DemonicCircleTeleport"] = 48020,
			["CreateHealthstone"] = 6201,
			["InternalCombustion"] = 266134,
			["Eradication"] = 196412,
			["Ruin"] = 387103,
			["ChaosIncarnate"] = 387275,
			["Lifeblood"] = 386646,
			["Conflagrate"] = 17962,
			["ImprovedImmolate"] = 387093,
			["ConflagrationofChaos"] = 387108,
			["GrimFeast"] = 386689,
			["SoulFire"] = 6353,
			["RoaringBlaze"] = 205184,
			["Mayhem"] = 387506,
			["RainofFire"] = 5740,
			["DimensionalRift"] = 387976,
			["BurntoAshes"] = 387153,
			["Soulstone"] = 20707,
			["CurseofWeakness"] = 702,
			["HealthFunnel"] = 755,
			["Incinerate"] = 29722,
			["AvatarofDestruction"] = 387159,
			["RitualofSummoning"] = 698,
			["Fear"] = 5782,
			["Backlash"] = 387384,
			["FireandBrimstone"] = 196408,
			["Darkfury"] = 264874,
			["SummonSoulkeeper"] = 386256,
			["UnendingResolve"] = 104773,
			["CommandDemon"] = 119898,
			["FelArmor"] = 386124,
			["Corruption"] = 172,
			},
			["Affliction"] = {
			["Soulburn"] = 385899,
			["SargereiTechnique"] = 405955,
			["Shadowflame"] = 384069,
			["UnendingBreath"] = 5697,
			["GrimoireofSynergy"] = 171975,
			["PandemicInvocation"] = 386759,
			["KindledMalice"] = 405330,
			["SoulFlame"] = 199471,
			["Horrify"] = 56244,
			["RitualofDoom"] = 342601,
			["SacrolashsDarkStrike"] = 386986,
			["AbsoluteCorruption"] = 196103,
			["GrimoireofSacrifice"] = 108503,
			["WrathfulMinion"] = 386864,
			["DoomBlossom"] = 416621,
			["MasteryPotentAfflictions"] = 77215,
			["SubjugateDemon"] = 1098,
			["AccruedVitality"] = 386613,
			["DemonicResilience"] = 389590,
			["MortalCoil"] = 6789,
			["GrandWarlocksDesign"] = 387084,
			["PhantomSingularity"] = 205179,
			["AmplifyCurse"] = 328774,
			["CursesofEnfeeblement"] = 386105,
			["WitheringBolt"] = 386976,
			["DemonSkin"] = 219272,
			["ProfaneBargain"] = 389576,
			["Agony"] = 980,
			["HowlofTerror"] = 5484,
			["SeizedVitality"] = 387250,
			["SoulConduit"] = 215941,
			["TheCodexofXerrath"] = 101508,
			["FelDomination"] = 333889,
			["SoulShards"] = 246985,
			["DreadTouch"] = 389775,
			["SoulLeech"] = 108370,
			["FelSynergy"] = 389367,
			["DemonicInspiration"] = 386858,
			["Shadowfury"] = 30283,
			["Darkfury"] = 264874,
			["Fear"] = 5782,
			["SoulLink"] = 108415,
			["CurseofWeakness"] = 702,
			["StrengthofWill"] = 317138,
			["FrequentDonor"] = 386686,
			["InquisitorsGaze"] = 386344,
			["Nightfall"] = 108558,
			["SoulEatersGluttony"] = 389630,
			["DrainSoul"] = 198590,
			["DrainLife"] = 234153,
			["TormentedCrescendo"] = 387075,
			["DarkVirtuosity"] = 405327,
			["DemonicCircle"] = 48018,
			["SweetSouls"] = 386620,
			["FelPact"] = 386113,
			["InevitableDemise"] = 334319,
			["Haunt"] = 48181,
			["SocretharsGuile"] = 405936,
			["CreepingDeath"] = 264000,
			["WritheinAgony"] = 196102,
			["CreateSoulwell"] = 29893,
			["VileTaint"] = 278350,
			["FiendishStride"] = 386110,
			["ShadowEmbrace"] = 32388,
			["ShadowBolt"] = 686,
			["CurseofExhaustion"] = 334275,
			["IchorofDevils"] = 386664,
			["ResoluteBarrier"] = 389359,
			["XavianTeachings"] = 317031,
			["SoulRot"] = 386997,
			["TeachingsoftheSatyr"] = 387972,
			["AgonizingCorruption"] = 386922,
			["FocusedMalignancy"] = 399668,
			["DemonicFortitude"] = 386617,
			["MalevolentVisionary"] = 387273,
			["WrathofConsumption"] = 387065,
			["SpellLock"] = 119910,
			["GrimReach"] = 389992,
			["BurningRush"] = 111400,
			["DemonicCircleTeleport"] = 48020,
			["CreateHealthstone"] = 6201,
			["TeachingsoftheBlackHarvest"] = 385881,
			["MaleficRapture"] = 324536,
			["XaviusGambit"] = 416615,
			["UnstableAffliction"] = 316099,
			["Lifeblood"] = 386646,
			["SiphonLife"] = 63106,
			["SowtheSeeds"] = 196226,
			["SeedofCorruption"] = 27243,
			["Nightmare"] = 386648,
			["SoulSwap"] = 386951,
			["DarkPact"] = 108416,
			["CurseofTongues"] = 1714,
			["HauntedSoul"] = 387301,
			["DarkHarvest"] = 387016,
			["DemonicGateway"] = 111771,
			["Banish"] = 710,
			["Corruption"] = 172,
			["HealthFunnel"] = 755,
			["GrimFeast"] = 386689,
			["AbyssWalker"] = 389609,
			["RitualofSummoning"] = 698,
			["EyeofKilrogg"] = 126,
			["DarkAccord"] = 386659,
			["Soulstone"] = 20707,
			["DemonicEmbrace"] = 288843,
			["SummonSoulkeeper"] = 386256,
			["UnendingResolve"] = 104773,
			["CommandDemon"] = 119898,
			["FelArmor"] = 386124,
			["SummonDarkglare"] = 205180,
			},
		},
		["Druid"] = {
			["Feral"] = {
			["Swiftmend"] = 18562,
			["MightyBash"] = 5211,
			["NurturingInstinct"] = 33873,
			["FranticMomentum"] = 391875,
			["BerserkHeartoftheLion"] = 391174,
			["IncessantTempest"] = 400140,
			["SouloftheForest"] = 158476,
			["Dreamwalk"] = 193753,
			["Mangle"] = 33917,
			["ImprovedBarkskin"] = 327993,
			["UrsineVigor"] = 377842,
			["AdaptiveSwarm"] = 391888,
			["Sabertooth"] = 202031,
			["WildGrowth"] = 48438,
			["Hibernate"] = 2637,
			["NaturesVigil"] = 124974,
			["RipandTear"] = 391347,
			["Starfire"] = 197628,
			["ImprovedSunfire"] = 231050,
			["MassEntanglement"] = 102359,
			["ThrashingClaws"] = 405300,
			["Predator"] = 202021,
			["CircleofLifeandDeath"] = 400320,
			["MercilessClaws"] = 231063,
			["Flap"] = 164862,
			["UrsolsVortex"] = 102793,
			["Soothe"] = 2908,
			["Thrash"] = 106832,
			["PouncingStrikes"] = 390772,
			["ImprovedSwipe"] = 400158,
			["Cyclone"] = 33786,
			["CatForm"] = 768,
			["OmenofClarity"] = 16864,
			["RemoveCorruption"] = 2782,
			["Regrowth"] = 8936,
			["AstralInfluence"] = 197524,
			["TigersTenacity"] = 391872,
			["FlightForm"] = 276029,
			["Barkskin"] = 22812,
			["WildCharge"] = 102401,
			["RagingFury"] = 391078,
			["BrutalSlash"] = 202028,
			["LunarInspiration"] = 155580,
			["BerserkFrenzy"] = 384668,
			["CharmWoodlandCreature"] = 127757,
			["Revive"] = 50769,
			["FelineAdept"] = 300349,
			["SuddenAmbush"] = 384667,
			["AquaticForm"] = 276012,
			["Rake"] = 1822,
			["Rip"] = 1079,
			["VerdantHeart"] = 301768,
			["KillerInstinct"] = 108299,
			["Berserk"] = 106951,
			["Starsurge"] = 197626,
			["MountForm"] = 210053,
			["ConvoketheSpirits"] = 391528,
			["Wrath"] = 5176,
			["Typhoon"] = 132469,
			["TirelessEnergy"] = 383352,
			["Sunfire"] = 93402,
			["SaberJaws"] = 421432,
			["Forestwalk"] = 400129,
			["IncapacitatingRoar"] = 99,
			["CarnivorousInstinct"] = 390902,
			["ProtectorofthePack"] = 378986,
			["FrenziedRegeneration"] = 22842,
			["PrimalWrath"] = 285381,
			["ImprovedStampedingRoar"] = 288826,
			["LionsStrength"] = 391972,
			["MattedFur"] = 385786,
			["ImprovedRejuvenation"] = 231040,
			["RampantFerocity"] = 391709,
			["FeralInstinct"] = 16949,
			["LycarasTeachings"] = 378988,
			["StampedingRoar"] = 106898,
			["ThickHide"] = 16931,
			["UnbridledSwarm"] = 391951,
			["IncarnationAvatarofAshamane"] = 102543,
			["AshamanesGuidance"] = 391548,
			["NaturalRecovery"] = 377796,
			["Shred"] = 5221,
			["DreadfulBleeding"] = 391045,
			["ApexPredatorsCraving"] = 391881,
			["TigerDash"] = 252216,
			["Rebirth"] = 20484,
			["TirelessPursuit"] = 377801,
			["Veinripper"] = 391978,
			["MasteryRazorClaws"] = 77493,
			["MoonkinForm"] = 197625,
			["MomentofClarity"] = 236068,
			["TasteforBlood"] = 384665,
			["TearOpenWounds"] = 391785,
			["SurvivalInstincts"] = 61336,
			["InfectedWounds"] = 48484,
			["RisingLightFallingNight"] = 417712,
			["Prowl"] = 5215,
			["PredatorySwiftness"] = 16974,
			["Ironfur"] = 192081,
			["TigersFury"] = 5217,
			["RelentlessPredator"] = 393771,
			["BearForm"] = 5487,
			["EntanglingRoots"] = 339,
			["Growl"] = 6795,
			["PrimalFury"] = 159286,
			["Renewal"] = 108238,
			["DoubleClawedRake"] = 391700,
			["FelineSwiftness"] = 131768,
			["Rejuvenation"] = 774,
			["HeartoftheWild"] = 319454,
			["Swipe"] = 213764,
			["Bloodtalons"] = 319439,
			["WildSlashes"] = 390864,
			["TreantForm"] = 114282,
			["Maim"] = 22570,
			["GaleWinds"] = 400142,
			["WellHonedInstincts"] = 377847,
			["Moonfire"] = 8921,
			["FeralFrenzy"] = 274837,
			["FerociousBite"] = 22568,
			["MarkoftheWild"] = 1126,
			["SkullBash"] = 106839,
			["DireFixation"] = 417710,
			["Dash"] = 1850,
			["Innervate"] = 29166,
			["ProtectiveGrowth"] = 391947,
			["TravelForm"] = 783,
			},
			["Balance"] = {
			["AstralSmolder"] = 394058,
			["NurturingInstinct"] = 33873,
			["MightyBash"] = 5211,
			["NaturesGrace"] = 393958,
			["SouloftheForest"] = 114107,
			["ImprovedBarkskin"] = 327993,
			["UrsineVigor"] = 377842,
			["Mangle"] = 33917,
			["SolarBeam"] = 78675,
			["WildGrowth"] = 48438,
			["Hibernate"] = 2637,
			["TravelForm"] = 783,
			["Starfire"] = 194153,
			["ImprovedSunfire"] = 231050,
			["MassEntanglement"] = 102359,
			["Renewal"] = 108238,
			["StellarFlare"] = 202347,
			["HeartoftheWild"] = 319454,
			["PrimalFury"] = 159286,
			["Dash"] = 1850,
			["AstralCommunion"] = 202359,
			["FerociousBite"] = 22568,
			["Thrash"] = 106832,
			["WellHonedInstincts"] = 377847,
			["FelineSwiftness"] = 131768,
			["Cyclone"] = 33786,
			["SkullBash"] = 106839,
			["MarkoftheWild"] = 1126,
			["Typhoon"] = 132469,
			["NaturesVigil"] = 124974,
			["AstralInfluence"] = 197524,
			["OrbitBreaker"] = 383197,
			["FlightForm"] = 276029,
			["SunderedFirmament"] = 394094,
			["WildCharge"] = 102401,
			["RadiantMoonlight"] = 394121,
			["IncarnationChosenofElune"] = 390414,
			["Solstice"] = 343647,
			["Soothe"] = 2908,
			["CharmWoodlandCreature"] = 127757,
			["Revive"] = 50769,
			["RemoveCorruption"] = 2782,
			["BalanceofAllThings"] = 394048,
			["AquaticForm"] = 276012,
			["UmbralIntensity"] = 383195,
			["Rip"] = 1079,
			["Starfall"] = 191034,
			["KillerInstinct"] = 108299,
			["ImprovedStampedingRoar"] = 288826,
			["Starsurge"] = 78674,
			["MountForm"] = 210053,
			["ConvoketheSpirits"] = 391528,
			["Wrath"] = 190984,
			["CatForm"] = 768,
			["Rejuvenation"] = 774,
			["Sunfire"] = 93402,
			["Regrowth"] = 8936,
			["Forestwalk"] = 400129,
			["Starlord"] = 202345,
			["WildSurges"] = 406890,
			["ProtectorofthePack"] = 378986,
			["FrenziedRegeneration"] = 22842,
			["LightoftheSun"] = 202918,
			["RattletheStars"] = 393954,
			["TwinMoons"] = 279620,
			["MattedFur"] = 385786,
			["Swiftmend"] = 18562,
			["IncapacitatingRoar"] = 99,
			["ImprovedSwipe"] = 400158,
			["WildMushroom"] = 88747,
			["StampedingRoar"] = 106898,
			["ForceofNature"] = 205636,
			["WaningTwilight"] = 393956,
			["MasteryAstralInvocation"] = 393014,
			["PowerofGoldrinn"] = 394046,
			["NaturalRecovery"] = 377796,
			["Shred"] = 5221,
			["EntanglingRoots"] = 339,
			["WarriorofElune"] = 202425,
			["TigerDash"] = 252216,
			["FriendoftheFae"] = 394081,
			["ThickHide"] = 16931,
			["MoonkinForm"] = 24858,
			["AetherialKindling"] = 327541,
			["Ironfur"] = 192081,
			["PrimordialArcanicPulsar"] = 393960,
			["StellarInnervation"] = 394115,
			["OrbitalStrike"] = 390378,
			["RisingLightFallingNight"] = 417712,
			["TirelessPursuit"] = 377801,
			["UmbralEmbrace"] = 393760,
			["Prowl"] = 5215,
			["CelestialAlignment"] = 194223,
			["NewMoon"] = 274281,
			["Flap"] = 164862,
			["Eclipse"] = 79577,
			["BearForm"] = 5487,
			["Rebirth"] = 20484,
			["Growl"] = 6795,
			["LunarShrapnel"] = 415169,
			["DenizenoftheDream"] = 394065,
			["CosmicRapidity"] = 400059,
			["VerdantHeart"] = 301768,
			["NaturesBalance"] = 202430,
			["Starweaver"] = 393940,
			["Swipe"] = 213764,
			["FuryofElune"] = 202770,
			["ElunesGuidance"] = 393991,
			["TreantForm"] = 114282,
			["Barkskin"] = 22812,
			["Dreamwalk"] = 193753,
			["ImprovedRejuvenation"] = 231040,
			["Moonfire"] = 8921,
			["ShootingStars"] = 202342,
			["LycarasTeachings"] = 378988,
			["Maim"] = 22570,
			["OwlkinFrenzy"] = 231042,
			["GaleWinds"] = 400142,
			["IncessantTempest"] = 400140,
			["Innervate"] = 29166,
			["Rake"] = 1822,
			["UrsolsVortex"] = 102793,
			},
			["Guardian"] = {
			["Swiftmend"] = 18562,
			["ElunesFavored"] = 370586,
			["ImprovedSurvivalInstincts"] = 328767,
			["ScintillatingMoonlight"] = 238049,
			["Dash"] = 1850,
			["TwinMoonfire"] = 372567,
			["ImprovedBarkskin"] = 327993,
			["UrsocsGuidance"] = 393414,
			["Mangle"] = 33917,
			["Gore"] = 210706,
			["WildGrowth"] = 48438,
			["Hibernate"] = 2637,
			["LayeredMane"] = 384721,
			["Thrash"] = 77758,
			["Starfire"] = 197628,
			["MoonlessNight"] = 400278,
			["MassEntanglement"] = 102359,
			["UrsocsFury"] = 377210,
			["Typhoon"] = 132469,
			["CircleofLifeandDeath"] = 391969,
			["VulnerableFlesh"] = 372618,
			["FuryofNature"] = 370695,
			["UrsolsVortex"] = 102793,
			["UntamedSavagery"] = 372943,
			["RendandTear"] = 204053,
			["Reinvigoration"] = 372945,
			["FelineSwiftness"] = 131768,
			["Cyclone"] = 33786,
			["SkullBash"] = 106839,
			["UrsineAdept"] = 300346,
			["Soothe"] = 2908,
			["NaturesVigil"] = 124974,
			["AstralInfluence"] = 197524,
			["BloodFrenzy"] = 203962,
			["FlightForm"] = 276029,
			["NurturingInstinct"] = 33873,
			["WildCharge"] = 102417,
			["CharmWoodlandCreature"] = 127757,
			["KillerInstinct"] = 108299,
			["VerdantHeart"] = 301768,
			["InnateResolve"] = 377811,
			["Earthwarden"] = 203974,
			["Revive"] = 50769,
			["RemoveCorruption"] = 2782,
			["Raze"] = 400254,
			["AquaticForm"] = 276012,
			["Rake"] = 1822,
			["Rip"] = 1079,
			["Pulverize"] = 80313,
			["MarkoftheWild"] = 1126,
			["Berserk"] = 50334,
			["Barkskin"] = 22812,
			["MountForm"] = 210053,
			["ConvoketheSpirits"] = 391528,
			["Wrath"] = 5176,
			["ImprovedSunfire"] = 231050,
			["Rejuvenation"] = 774,
			["Sunfire"] = 93402,
			["GaleWinds"] = 400142,
			["Forestwalk"] = 400129,
			["IncapacitatingRoar"] = 99,
			["FlashingClaws"] = 393427,
			["ProtectorofthePack"] = 378986,
			["FrenziedRegeneration"] = 22842,
			["LightningReflexes"] = 231065,
			["ImprovedStampedingRoar"] = 288826,
			["ThornsofIron"] = 400222,
			["MattedFur"] = 385786,
			["HeartoftheWild"] = 319454,
			["ThickHide"] = 16931,
			["GalacticGuardian"] = 203964,
			["Renewal"] = 108238,
			["StampedingRoar"] = 77761,
			["ImprovedRejuvenation"] = 231040,
			["PrimalFury"] = 159286,
			["MightyBash"] = 5211,
			["TigerDash"] = 252216,
			["NaturalRecovery"] = 377796,
			["Shred"] = 5221,
			["EntanglingRoots"] = 339,
			["SurvivalInstincts"] = 61336,
			["GuardianofElune"] = 155578,
			["Rebirth"] = 20484,
			["UrsineVigor"] = 377842,
			["IncarnationGuardianofUrsoc"] = 102558,
			["BristlingFur"] = 155835,
			["MoonkinForm"] = 197625,
			["SurvivaloftheFittest"] = 203965,
			["GoryFur"] = 200854,
			["AftertheWildfire"] = 371905,
			["Ironfur"] = 192081,
			["InfectedWounds"] = 345208,
			["RisingLightFallingNight"] = 417712,
			["Prowl"] = 5215,
			["TirelessPursuit"] = 377801,
			["Flap"] = 164862,
			["SouloftheForest"] = 158477,
			["DreamofCenarius"] = 372119,
			["BearForm"] = 5487,
			["ViciousCycle"] = 371999,
			["Growl"] = 6795,
			["UrsocsEndurance"] = 393611,
			["LunarBeam"] = 204066,
			["MasteryNaturesGuardian"] = 155783,
			["Maul"] = 6807,
			["RageoftheSleeper"] = 200851,
			["Starsurge"] = 197626,
			["Swipe"] = 213771,
			["Maim"] = 22570,
			["ToothandClaw"] = 135288,
			["TreantForm"] = 114282,
			["CatForm"] = 768,
			["Dreamwalk"] = 193753,
			["WellHonedInstincts"] = 377847,
			["Moonfire"] = 8921,
			["Regrowth"] = 8936,
			["FerociousBite"] = 22568,
			["Brambles"] = 203953,
			["ImprovedSwipe"] = 400158,
			["LycarasTeachings"] = 378988,
			["IncessantTempest"] = 400140,
			["Innervate"] = 29166,
			["ReinforcedFur"] = 393618,
			["TravelForm"] = 783,
			},
			["Restoration"] = {
			["Swiftmend"] = 18562,
			["MightyBash"] = 5211,
			["WildSynthesis"] = 400533,
			["SouloftheForest"] = 158478,
			["CenariusGuidance"] = 393371,
			["ImprovedBarkskin"] = 327993,
			["UrsineVigor"] = 377842,
			["AdaptiveSwarm"] = 391888,
			["Reforestation"] = 392356,
			["Hibernate"] = 2637,
			["HeartoftheWild"] = 319454,
			["Thrash"] = 106832,
			["ImprovedSwipe"] = 400158,
			["Cyclone"] = 33786,
			["SkullBash"] = 106839,
			["PassingSeasons"] = 382550,
			["NaturesVigil"] = 124974,
			["AstralInfluence"] = 197524,
			["FlightForm"] = 276029,
			["WildCharge"] = 102401,
			["CharmWoodlandCreature"] = 127757,
			["RemoveCorruption"] = 440015,
			["AquaticForm"] = 276012,
			["HarmoniousBlooming"] = 392256,
			["Maim"] = 22570,
			["Starsurge"] = 197626,
			["MountForm"] = 210053,
			["BuddingLeaves"] = 392167,
			["Sunfire"] = 93402,
			["Regenesis"] = 383191,
			["IncapacitatingRoar"] = 99,
			["Photosynthesis"] = 274902,
			["RegenerativeHeartwood"] = 392116,
			["MasteryHarmony"] = 77495,
			["MattedFur"] = 385786,
			["StampedingRoar"] = 106898,
			["Efflorescence"] = 145205,
			["UnbridledSwarm"] = 391951,
			["TranquilMind"] = 403521,
			["TigerDash"] = 252216,
			["Abundance"] = 207383,
			["Nourish"] = 50464,
			["ThickHide"] = 16931,
			["UnstoppableGrowth"] = 382559,
			["Swipe"] = 213764,
			["Verdancy"] = 392325,
			["Liveliness"] = 426702,
			["Cultivation"] = 200390,
			["Moonfire"] = 8921,
			["FerociousBite"] = 22568,
			["NaturesSplendor"] = 392288,
			["Ironbark"] = 102342,
			["VerdantInfusion"] = 392410,
			["TravelForm"] = 783,
			["Mangle"] = 33917,
			["WildGrowth"] = 48438,
			["Stonebark"] = 197061,
			["ImprovedNaturesCure"] = 392378,
			["RampantGrowth"] = 404521,
			["Starfire"] = 197628,
			["ImprovedSunfire"] = 231050,
			["MassEntanglement"] = 102359,
			["ImprovedRegrowth"] = 231032,
			["FlashofClarity"] = 392220,
			["UrsolsVortex"] = 102793,
			["GroveTending"] = 383192,
			["PrimalFury"] = 159286,
			["CenarionWard"] = 102351,
			["Regrowth"] = 8936,
			["NurturingDormancy"] = 392099,
			["Overgrowth"] = 203651,
			["Invigorate"] = 392160,
			["YserasGift"] = 145108,
			["GaleWinds"] = 400142,
			["Rake"] = 1822,
			["Rip"] = 1079,
			["EmbraceoftheDream"] = 392124,
			["KillerInstinct"] = 108299,
			["Barkskin"] = 22812,
			["ImprovedStampedingRoar"] = 288826,
			["Wrath"] = 5176,
			["Rejuvenation"] = 774,
			["ProtectorofthePack"] = 378986,
			["FrenziedRegeneration"] = 22842,
			["Undergrowth"] = 392301,
			["Revitalize"] = 212040,
			["IncarnationTreeofLife"] = 33891,
			["VerdantHeart"] = 301768,
			["NaturalRecovery"] = 377796,
			["Shred"] = 5221,
			["EntanglingRoots"] = 339,
			["Revive"] = 50769,
			["Innervate"] = 29166,
			["Rebirth"] = 20484,
			["WellHonedInstincts"] = 377847,
			["ImprovedIronbark"] = 382552,
			["MoonkinForm"] = 197625,
			["Ironfur"] = 192081,
			["Typhoon"] = 132469,
			["IncessantTempest"] = 400140,
			["MasterShapeshifter"] = 289237,
			["WakingDream"] = 392221,
			["Prowl"] = 5215,
			["RisingLightFallingNight"] = 417712,
			["Flourish"] = 197721,
			["ImprovedRejuvenation"] = 231040,
			["Tranquility"] = 740,
			["PoweroftheArchdruid"] = 392302,
			["Dreamwalk"] = 193753,
			["BearForm"] = 5487,
			["NaturesSwiftness"] = 132158,
			["Lifebloom"] = 188550,
			["FelineSwiftness"] = 131768,
			["GroveGuardians"] = 102693,
			["Dreamstate"] = 392162,
			["ImprovedWildGrowth"] = 328025,
			["Forestwalk"] = 400129,
			["ConvoketheSpirits"] = 391528,
			["OmenofClarity"] = 113043,
			["CatForm"] = 768,
			["LuxuriantSoil"] = 392315,
			["TreantForm"] = 114282,
			["Growl"] = 6795,
			["TirelessPursuit"] = 377801,
			["Soothe"] = 2908,
			["Flap"] = 164862,
			["InnerPeace"] = 197073,
			["Renewal"] = 108238,
			["Germination"] = 155675,
			["MarkoftheWild"] = 1126,
			["SpringBlossoms"] = 207385,
			["Dash"] = 1850,
			["LycarasTeachings"] = 378988,
			["NurturingInstinct"] = 33873,
			["NaturesCure"] = 88423,
			},
		},
		["Priest"] = {
			["Shadow"] = {
			["Renew"] = 139,
			["VoidTorrent"] = 263165,
			["DistortedReality"] = 409044,
			["MentalDecay"] = 375994,
			["DarkEvangelism"] = 391095,
			["MindControl"] = 605,
			["PowerWordShield"] = 17,
			["IdolofYoggSaron"] = 373273,
			["ScreamsoftheVoid"] = 375767,
			["Mindgames"] = 375901,
			["TwistofFate"] = 390972,
			["MindSpike"] = 73510,
			["ThroesofPain"] = 377422,
			["SurgeofInsanity"] = 391399,
			["PsychicVoice"] = 196704,
			["MindsEye"] = 407470,
			["ProtectiveLight"] = 193063,
			["AuspiciousSpirits"] = 155271,
			["Hallucinations"] = 280752,
			["BodyandSoul"] = 64129,
			["PhantasmalPathogen"] = 407469,
			["Mastermind"] = 391151,
			["Manipulation"] = 390996,
			["BindingHeals"] = 368275,
			["MentalFortitude"] = 377065,
			["EssenceDevourer"] = 415479,
			["Benevolence"] = 415416,
			["Resurrection"] = 2006,
			["PsychicLink"] = 199484,
			["Mindbender"] = 200174,
			["Rhapsody"] = 390622,
			["AncientMadness"] = 341240,
			["TormentedSpirits"] = 391284,
			["ImprovedFade"] = 390670,
			["MaddeningTouch"] = 391228,
			["VampiricEmbrace"] = 15286,
			["Shadowform"] = 232698,
			["Deathspeaker"] = 392507,
			["Voidform"] = 228264,
			["MindSoothe"] = 453,
			["MindVision"] = 2096,
			["ImprovedFlashHeal"] = 393870,
			["SanguineTeachings"] = 373218,
			["WordsofthePious"] = 377438,
			["VoidShield"] = 280749,
			["SheerTerror"] = 390919,
			["MasteryShadowWeaving"] = 343690,
			["MindDevourer"] = 373202,
			["ShackleUndead"] = 9484,
			["VoidShift"] = 108968,
			["FromDarknessComesLight"] = 390615,
			["BlessedRecovery"] = 390767,
			["FocusedMending"] = 372354,
			["DesperatePrayer"] = 19236,
			["Malediction"] = 373221,
			["ShatteredPerceptions"] = 391112,
			["PsychicScream"] = 8122,
			["PurifyDisease"] = 213634,
			["PrayerofMending"] = 33076,
			["MentalAgility"] = 341167,
			["DispelMagic"] = 528,
			["ShadowyInsight"] = 375888,
			["TwinsoftheSunPriestess"] = 373466,
			["Silence"] = 15487,
			["IdolofCThun"] = 377349,
			["Fade"] = 586,
			["Sanlayn"] = 199855,
			["Apathy"] = 390668,
			["DevouringPlague"] = 335467,
			["ShadowCrash"] = 205385,
			["ShadowWordPain"] = 589,
			["PowerInfusion"] = 10060,
			["InsidiousIre"] = 373212,
			["FocusedWill"] = 45243,
			["FlashHeal"] = 2061,
			["UnwaveringWill"] = 373456,
			["Levitate"] = 1706,
			["DominateMind"] = 205364,
			["Halo"] = 120644,
			["VoidTendrils"] = 108920,
			["UnfurlingDarkness"] = 341273,
			["AngelicFeather"] = 121536,
			["Inspiration"] = 390676,
			["AngelsMercy"] = 238100,
			["DarkAscension"] = 391109,
			["Smite"] = 585,
			["PetrifyingScream"] = 55676,
			["WhisperingShadows"] = 406777,
			["LightsInspiration"] = 373450,
			["LeapofFaith"] = 73325,
			["LastWord"] = 263716,
			["VoidBolt"] = 228266,
			["HolyNova"] = 132157,
			["PowerWordFortitude"] = 21562,
			["IdolofNZoth"] = 373280,
			["PowerWordLife"] = 373481,
			["MindMelt"] = 391090,
			["IdolofYShaarj"] = 373310,
			["Intangibility"] = 288733,
			["VoidEruption"] = 228260,
			["ThoughtHarvester"] = 406788,
			["ShadowWordDeath"] = 32379,
			["VampiricTouch"] = 34914,
			["CrystallineReflection"] = 373457,
			["DeathandMadness"] = 321291,
			["Dispersion"] = 47585,
			["DivineStar"] = 122121,
			["MindBlast"] = 8092,
			["Misery"] = 238558,
			["TranslucentImage"] = 373446,
			["InescapableTorment"] = 373427,
			["AngelicBulwark"] = 108945,
			["PsychicHorror"] = 64044,
			["MovewithGrace"] = 390620,
			["Phantasm"] = 108942,
			["MassDispel"] = 32375,
			["ShadowyApparitions"] = 341491,
			["SurgeofLight"] = 109186,
			["Shadowfiend"] = 34433,
			["Voidtouched"] = 407430,
			["TitheEvasion"] = 373223,
			["SpellWarding"] = 390667,
			},
			["Holy"] = {
			["Renew"] = 139,
			["PrayerofMending"] = 33076,
			["Pontifex"] = 390980,
			["GuardianAngel"] = 200209,
			["Orison"] = 390947,
			["DesperateTimes"] = 391381,
			["HolyWordSalvation"] = 265202,
			["Heal"] = 2060,
			["VoidTendrils"] = 108920,
			["HolyWordSerenity"] = 2050,
			["MindControl"] = 605,
			["PowerWordShield"] = 17,
			["Benediction"] = 193157,
			["WordsofthePious"] = 377438,
			["MasteryEchoofLight"] = 77485,
			["HolyMending"] = 391154,
			["SurgeofLight"] = 109186,
			["SpiritofRedemption"] = 20711,
			["ThroesofPain"] = 377422,
			["PrayerfulLitany"] = 391209,
			["Manipulation"] = 390996,
			["EssenceDevourer"] = 415479,
			["ProtectiveLight"] = 193063,
			["Resurrection"] = 2006,
			["BindingHeals"] = 368275,
			["DivineImage"] = 392988,
			["SanctifiedPrayers"] = 196489,
			["SanguineTeachings"] = 373218,
			["AngelicBulwark"] = 108945,
			["DivineHymn"] = 64843,
			["VoidShield"] = 280749,
			["DivineWord"] = 372760,
			["Benevolence"] = 415416,
			["DispelMagic"] = 528,
			["TwistofFate"] = 390972,
			["PrismaticEchoes"] = 390967,
			["DesperatePrayer"] = 19236,
			["SymbolofHope"] = 64901,
			["SheerTerror"] = 390919,
			["ImprovedFade"] = 390670,
			["PrayerCircle"] = 321377,
			["Censure"] = 200199,
			["TranslucentImage"] = 373446,
			["ShadowWordDeath"] = 32379,
			["MassResurrection"] = 212036,
			["MindSoothe"] = 453,
			["Lightwell"] = 372835,
			["ImprovedFlashHeal"] = 393870,
			["MindVision"] = 2096,
			["HealingChorus"] = 390881,
			["DeathandMadness"] = 321291,
			["FocusedMending"] = 372354,
			["BodyandSoul"] = 64129,
			["Enlightenment"] = 193155,
			["Apotheosis"] = 200183,
			["Lightweaver"] = 390992,
			["FromDarknessComesLight"] = 390615,
			["BlessedRecovery"] = 390767,
			["Apathy"] = 390668,
			["VoidShift"] = 108968,
			["GalesofSong"] = 372370,
			["TrailofLight"] = 200128,
			["AngelicFeather"] = 121536,
			["PurifyDisease"] = 440006,
			["GuardianSpirit"] = 47788,
			["EmpyrealBlaze"] = 372616,
			["MassDispel"] = 32375,
			["MentalAgility"] = 341167,
			["TwinsoftheSunPriestess"] = 373466,
			["Purify"] = 527,
			["DominateMind"] = 205364,
			["Fade"] = 586,
			["Sanlayn"] = 199855,
			["UnwaveringWill"] = 373456,
			["Levitate"] = 1706,
			["SayYourPrayers"] = 391186,
			["DivineStar"] = 110744,
			["LightoftheNaaru"] = 196985,
			["AnsweredPrayers"] = 391387,
			["FocusedWill"] = 45243,
			["LeapofFaith"] = 73325,
			["RevitalizingPrayers"] = 391208,
			["PrayersoftheVirtuous"] = 390977,
			["FlashHeal"] = 2061,
			["Halo"] = 120517,
			["PetrifyingScream"] = 55676,
			["PowerInfusion"] = 10060,
			["Smite"] = 585,
			["Inspiration"] = 390676,
			["AngelsMercy"] = 238100,
			["PrayerofHealing"] = 596,
			["VoiceofHarmony"] = 390994,
			["Afterlife"] = 196707,
			["HolyWordChastise"] = 88625,
			["LightsInspiration"] = 373450,
			["BurningVehemence"] = 372307,
			["ShackleUndead"] = 9484,
			["CosmicRipple"] = 238136,
			["HolyNova"] = 132157,
			["PowerWordFortitude"] = 21562,
			["Epiphany"] = 414553,
			["PowerWordLife"] = 373481,
			["ShadowWordPain"] = 589,
			["MiracleWorker"] = 235587,
			["GuardiansoftheLight"] = 196437,
			["EverlastingLight"] = 391161,
			["Rhapsody"] = 390622,
			["ResonantWords"] = 372309,
			["CrisisManagement"] = 390954,
			["CrystallineReflection"] = 373457,
			["ShatteredPerceptions"] = 391112,
			["ImprovedPurify"] = 390632,
			["Mindgames"] = 375901,
			["MindBlast"] = 8092,
			["PsychicScream"] = 8122,
			["DivineService"] = 391233,
			["HolyWordSanctify"] = 34861,
			["VampiricEmbrace"] = 15286,
			["CircleofHealing"] = 204883,
			["MovewithGrace"] = 390620,
			["Phantasm"] = 108942,
			["HolyFire"] = 14914,
			["RenewedFaith"] = 341997,
			["Restitution"] = 391124,
			["Shadowfiend"] = 34433,
			["PsychicVoice"] = 196704,
			["TitheEvasion"] = 373223,
			["SpellWarding"] = 390667,
			},
			["Discipline"] = {
			["Renew"] = 139,
			["PrayerofMending"] = 33076,
			["DarkIndulgence"] = 372972,
			["MindControl"] = 605,
			["VoidTendrils"] = 108920,
			["PainandSuffering"] = 390689,
			["EnduringLuminescence"] = 390685,
			["PowerWordShield"] = 17,
			["ProtectiveLight"] = 193063,
			["WordsofthePious"] = 377438,
			["AbyssalReverie"] = 373054,
			["Mindgames"] = 375901,
			["PsychicVoice"] = 196704,
			["InescapableTorment"] = 373427,
			["PowerWordRadiance"] = 194509,
			["ShadowCovenant"] = 314867,
			["BodyandSoul"] = 64129,
			["ProtectoroftheFrail"] = 373035,
			["Exaltation"] = 373042,
			["LuminousBarrier"] = 271466,
			["BindingHeals"] = 368275,
			["VoidSummoner"] = 390770,
			["Schism"] = 424509,
			["AegisofWrath"] = 238135,
			["Manipulation"] = 390996,
			["DesperatePrayer"] = 19236,
			["UltimatePenitence"] = 421453,
			["EssenceDevourer"] = 415479,
			["Benevolence"] = 415416,
			["DispelMagic"] = 528,
			["OverloadedwithLight"] = 421557,
			["Mindbender"] = 123040,
			["Rhapsody"] = 390622,
			["PainSuppression"] = 33206,
			["BlazeofLight"] = 215768,
			["ImprovedFade"] = 390670,
			["MassDispel"] = 32375,
			["VampiricEmbrace"] = 15286,
			["SheerTerror"] = 390919,
			["Expiation"] = 390832,
			["MassResurrection"] = 212036,
			["MindSoothe"] = 453,
			["DivineAegis"] = 47515,
			["ImprovedFlashHeal"] = 393870,
			["MindVision"] = 2096,
			["MasteryGrace"] = 271534,
			["VoidShield"] = 280749,
			["FocusedMending"] = 372354,
			["SinsoftheMany"] = 280391,
			["Castigation"] = 193134,
			["PoweroftheDarkSide"] = 198068,
			["VoidShift"] = 108968,
			["ImprovedPurify"] = 390632,
			["BlessedRecovery"] = 390767,
			["Lenience"] = 238063,
			["HeavensWrath"] = 421558,
			["DeathandMadness"] = 321291,
			["ShatteredPerceptions"] = 391112,
			["Atonement"] = 81749,
			["PurifyDisease"] = 440006,
			["Evangelism"] = 246287,
			["MentalAgility"] = 341167,
			["MaliciousIntent"] = 372969,
			["Rapture"] = 47536,
			["TwinsoftheSunPriestess"] = 373466,
			["FromDarknessComesLight"] = 390615,
			["BrightPupil"] = 390684,
			["Fade"] = 586,
			["Sanlayn"] = 199855,
			["Apathy"] = 390668,
			["TrainofThought"] = 390693,
			["BorrowedTime"] = 390691,
			["DivineStar"] = 110744,
			["PowerInfusion"] = 10060,
			["RevelinPurity"] = 373003,
			["FocusedWill"] = 45243,
			["AngelicFeather"] = 121536,
			["HarshDiscipline"] = 373180,
			["Purify"] = 527,
			["WealandWoe"] = 390786,
			["Halo"] = 120517,
			["AngelicBulwark"] = 108945,
			["ShackleUndead"] = 9484,
			["Sanctuary"] = 231682,
			["ShieldDiscipline"] = 197045,
			["AngelsMercy"] = 238100,
			["UnwaveringWill"] = 373456,
			["Smite"] = 585,
			["HolyNova"] = 132157,
			["Contrition"] = 197419,
			["LightsInspiration"] = 373450,
			["PainTransformation"] = 372991,
			["PainfulPunishment"] = 390686,
			["PetrifyingScream"] = 55676,
			["LeapofFaith"] = 73325,
			["PowerWordFortitude"] = 21562,
			["Inspiration"] = 390676,
			["PowerWordLife"] = 373481,
			["Indemnity"] = 373049,
			["ThroesofPain"] = 377422,
			["DominateMind"] = 205364,
			["Levitate"] = 1706,
			["FlashHeal"] = 2061,
			["PurgetheWicked"] = 204197,
			["ShadowWordPain"] = 589,
			["CrystallineReflection"] = 373457,
			["TranslucentImage"] = 373446,
			["TwilightEquilibrium"] = 390705,
			["TwistofFate"] = 390972,
			["MindBlast"] = 8092,
			["PsychicScream"] = 8122,
			["PowerWordBarrier"] = 62618,
			["SanguineTeachings"] = 373218,
			["ShadowWordDeath"] = 32379,
			["TwilightCorruption"] = 373065,
			["MovewithGrace"] = 390620,
			["Phantasm"] = 108942,
			["Resurrection"] = 2006,
			["Penance"] = 47540,
			["SurgeofLight"] = 109186,
			["Shadowfiend"] = 34433,
			["LightsPromise"] = 322115,
			["TitheEvasion"] = 373223,
			["SpellWarding"] = 390667,
			},
		},
		["Hunter"] = {
			["Marksmanship"] = {
			["PoisonInjection"] = 378014,
			["NaturesEndurance"] = 388042,
			["BindingShackles"] = 321468,
			["WildernessMedicine"] = 343242,
			["WailingArrow"] = 392060,
			["IntheRhythm"] = 407404,
			["WindrunnersBarrage"] = 389866,
			["AimedShot"] = 19434,
			["LegacyoftheWindrunners"] = 406425,
			["ImprovedKillShot"] = 343248,
			["ScareBeast"] = 1513,
			["MasterMarksman"] = 260309,
			["Volley"] = 260243,
			["SentinelsProtection"] = 388057,
			["TargetPractice"] = 321287,
			["ConcussiveShot"] = 5116,
			["TrickShots"] = 257621,
			["ArcaneShot"] = 185358,
			["CrackShot"] = 321293,
			["Barrage"] = 120360,
			["Streamline"] = 260367,
			["Readiness"] = 389865,
			["MasterySniperTraining"] = 193468,
			["MultiShot"] = 257620,
			["TacticalReload"] = 400472,
			["HuntersAvoidance"] = 384799,
			["TranquilizingShot"] = 19801,
			["Salvo"] = 400456,
			["Pathfinding"] = 378002,
			["WindrunnersGuidance"] = 378905,
			["SentinelsPerception"] = 388056,
			["ImprovedTraps"] = 343247,
			["Camouflage"] = 199483,
			["WingClip"] = 195645,
			["KillerInstinct"] = 273887,
			["SteadyShot"] = 56641,
			["Intimidation"] = 19577,
			["RazorFragments"] = 384790,
			["CallingtheShots"] = 260404,
			["Exhilaration"] = 109304,
			["SurgingShots"] = 391559,
			["HuntersMark"] = 257284,
			["AlphaPredator"] = 269737,
			["Sharpshooter"] = 378907,
			["SteadyFocus"] = 193533,
			["RoarofSacrifice"] = 53480,
			["FocusedAim"] = 378767,
			["HuntersKnowledge"] = 378766,
			["FreezingTrap"] = 187650,
			["PreciseShots"] = 260240,
			["EagleEye"] = 6197,
			["UnerringVision"] = 386878,
			["Entrapment"] = 393344,
			["ImprovedSteadyShot"] = 321018,
			["AspectoftheCheetah"] = 186257,
			["LoneSurvivor"] = 388039,
			["SerpentstalkersTrickery"] = 378888,
			["KillShot"] = 53351,
			["CommandPet"] = 272651,
			["SerpentSting"] = 271788,
			["KeenEyesight"] = 378004,
			["KillerAccuracy"] = 378765,
			["AspectoftheTurtle"] = 186265,
			["BornToBeWild"] = 266921,
			["Bulletstorm"] = 389019,
			["Bullseye"] = 204089,
			["ImprovedTranquilizingShot"] = 343244,
			["LightAmmo"] = 378913,
			["TarTrap"] = 187698,
			["CarefulAim"] = 260228,
			["KillCommand"] = 34026,
			["BeastMaster"] = 378007,
			["SurvivaloftheFittest"] = 264735,
			["BindingShot"] = 109248,
			["DeathChakram"] = 375891,
			["LoneWolf"] = 155228,
			["LockandLoad"] = 194595,
			["ImprovedKillCommand"] = 378010,
			["Bombardment"] = 405804,
			["Deadeye"] = 321460,
			["NaturalMending"] = 270581,
			["Misdirection"] = 34477,
			["Posthaste"] = 109215,
			["Stampede"] = 201430,
			["EagletalonsTrueFocus"] = 389449,
			["SentinelOwl"] = 388045,
			["CounterShot"] = 147362,
			["ChimaeraShot"] = 342049,
			["SerratedShots"] = 389882,
			["RejuvenatingWind"] = 385539,
			["BurstingShot"] = 186387,
			["QuickLoad"] = 378771,
			["FeignDeath"] = 5384,
			["Disengage"] = 781,
			["Trailblazer"] = 199921,
			["Trueshot"] = 288613,
			["HighExplosiveTrap"] = 236776,
			["ScatterShot"] = 213691,
			["SteelTrap"] = 162488,
			["EyesoftheBeast"] = 321297,
			["Flare"] = 1543,
			["HydrasBite"] = 260241,
			["Deathblow"] = 378769,
			["ArcticBola"] = 390231,
			["RapidFire"] = 257044,
			["Fetch"] = 125050,
			["ExplosiveShot"] = 212431,
			["HeavyAmmo"] = 378910,
			["PlayDead"] = 209997,
			},
			["Beast Mastery"] = {
			["PrimalRage"] = 272678,
			["PoisonInjection"] = 378014,
			["NaturesEndurance"] = 388042,
			["BrutalCompanion"] = 386870,
			["BeastCleave"] = 115939,
			["BindingShackles"] = 321468,
			["WildernessMedicine"] = 343242,
			["WailingArrow"] = 392060,
			["Fetch"] = 125050,
			["ImprovedKillShot"] = 343248,
			["ScareBeast"] = 1513,
			["MasterMarksman"] = 260309,
			["KindredSpirits"] = 56315,
			["SteelTrap"] = 162488,
			["SentinelsProtection"] = 388057,
			["DireFrenzy"] = 385810,
			["ConcussiveShot"] = 5116,
			["ScatterShot"] = 213691,
			["ThrilloftheHunt"] = 257944,
			["Posthaste"] = 109215,
			["Barrage"] = 120360,
			["MasterHandler"] = 424558,
			["SharpBarbs"] = 378205,
			["MultiShot"] = 2643,
			["Misdirection"] = 34477,
			["TranquilizingShot"] = 19801,
			["AMurderofCrows"] = 131894,
			["Pathfinding"] = 378002,
			["WildInstincts"] = 378442,
			["SentinelsPerception"] = 388056,
			["ImprovedTraps"] = 343247,
			["WingClip"] = 195645,
			["KillerInstinct"] = 273887,
			["Intimidation"] = 19577,
			["MasteryMasterofBeasts"] = 76657,
			["SteadyShot"] = 56641,
			["Exhilaration"] = 109304,
			["CobraSenses"] = 378244,
			["KillCleave"] = 378207,
			["Flare"] = 1543,
			["Stampede"] = 201430,
			["ArcaneShot"] = 185358,
			["RoarofSacrifice"] = 53480,
			["FreezingTrap"] = 187650,
			["KillCommand"] = 34026,
			["Savagery"] = 424557,
			["AspectoftheTurtle"] = 186265,
			["KillerCobra"] = 199532,
			["AspectoftheBeast"] = 191384,
			["Entrapment"] = 393344,
			["LoneSurvivor"] = 388039,
			["AspectoftheCheetah"] = 186257,
			["EagleEye"] = 6197,
			["CalloftheWild"] = 359844,
			["KillShot"] = 53351,
			["ScentofBlood"] = 193532,
			["SerpentSting"] = 271788,
			["KeenEyesight"] = 378004,
			["Trailblazer"] = 199921,
			["Stomp"] = 199530,
			["TarTrap"] = 187698,
			["CobraShot"] = 193455,
			["NaturalMending"] = 270581,
			["Bloodshed"] = 321530,
			["BornToBeWild"] = 266921,
			["ExoticBeasts"] = 53270,
			["HuntersMark"] = 257284,
			["Disengage"] = 781,
			["FeignDeath"] = 5384,
			["SurvivaloftheFittest"] = 264735,
			["BindingShot"] = 109248,
			["DeathChakram"] = 375891,
			["PiercingFangs"] = 392053,
			["OnewiththePack"] = 199528,
			["ImprovedKillCommand"] = 378010,
			["BarbedWrath"] = 231548,
			["SerratedShots"] = 389882,
			["PackTactics"] = 321014,
			["AnimalCompanion"] = 267116,
			["CobraSting"] = 378750,
			["WarOrders"] = 393933,
			["DireCommand"] = 378743,
			["SentinelOwl"] = 388045,
			["CounterShot"] = 147362,
			["Camouflage"] = 199483,
			["BarbedShot"] = 217200,
			["RejuvenatingWind"] = 385539,
			["BloodyFrenzy"] = 407412,
			["PlayDead"] = 209997,
			["BestialWrath"] = 19574,
			["DireBeast"] = 120679,
			["DirePack"] = 378745,
			["KillerCommand"] = 378740,
			["HighExplosiveTrap"] = 236776,
			["ImprovedTranquilizingShot"] = 343244,
			["WildCall"] = 185789,
			["EyesoftheBeast"] = 321297,
			["CommandPet"] = 272651,
			["HydrasBite"] = 260241,
			["HuntersPrey"] = 378210,
			["ArcticBola"] = 390231,
			["AlphaPredator"] = 269737,
			["HuntersAvoidance"] = 384799,
			["ExplosiveShot"] = 212431,
			["BeastMaster"] = 378007,
			["TrainingExpert"] = 378209,
			},
			["Survival"] = {
			["PoisonInjection"] = 378014,
			["NaturesEndurance"] = 388042,
			["BindingShackles"] = 321468,
			["WildernessMedicine"] = 343242,
			["ImprovedWildfireBomb"] = 321290,
			["Fetch"] = 125050,
			["ImprovedKillShot"] = 343248,
			["ScareBeast"] = 1513,
			["MasterMarksman"] = 260309,
			["SteelTrap"] = 162488,
			["SentinelsProtection"] = 388057,
			["SweepingSpear"] = 378950,
			["ConcussiveShot"] = 5116,
			["ScatterShot"] = 213691,
			["ArcaneShot"] = 185358,
			["Posthaste"] = 109215,
			["RuthlessMarauder"] = 385718,
			["Bloodseeker"] = 260248,
			["WildfireInfusion"] = 271014,
			["TacticalAdvantage"] = 378951,
			["CoordinatedAssault"] = 360952,
			["Stampede"] = 201430,
			["BirdsofPrey"] = 260331,
			["KillerCompanion"] = 378955,
			["TranquilizingShot"] = 19801,
			["Pathfinding"] = 378002,
			["SentinelsPerception"] = 388056,
			["MongooseBite"] = 259387,
			["Camouflage"] = 199483,
			["WingClip"] = 195645,
			["KillerInstinct"] = 273887,
			["SteadyShot"] = 56641,
			["Intimidation"] = 19577,
			["PlayDead"] = 209997,
			["HuntersMark"] = 257284,
			["Exhilaration"] = 109304,
			["VipersVenom"] = 268501,
			["MasterySpiritBond"] = 263135,
			["Flare"] = 1543,
			["SpearFocus"] = 378953,
			["FlankingStrike"] = 269751,
			["RoarofSacrifice"] = 53480,
			["LoneSurvivor"] = 388039,
			["KillCommand"] = 259489,
			["FreezingTrap"] = 187650,
			["QuickShot"] = 378940,
			["EagleEye"] = 6197,
			["TarTrap"] = 187698,
			["Entrapment"] = 393344,
			["Muzzle"] = 187707,
			["AspectoftheCheetah"] = 186257,
			["CommandPet"] = 272651,
			["ShrapnelBomb"] = 270335,
			["Butchery"] = 212436,
			["Spearhead"] = 360966,
			["SerpentSting"] = 271788,
			["KeenEyesight"] = 378004,
			["Trailblazer"] = 199921,
			["HighExplosiveTrap"] = 236776,
			["Bombardier"] = 389880,
			["SharpEdges"] = 378948,
			["NaturalMending"] = 270581,
			["ImprovedTranquilizingShot"] = 343244,
			["FrenzyStrikes"] = 294029,
			["BornToBeWild"] = 266921,
			["Carve"] = 187708,
			["Disengage"] = 781,
			["AspectoftheEagle"] = 186289,
			["SurvivaloftheFittest"] = 264735,
			["BindingShot"] = 109248,
			["DeathChakram"] = 375891,
			["Misdirection"] = 34477,
			["IntenseFocus"] = 385709,
			["ImprovedKillCommand"] = 378010,
			["Lunge"] = 378934,
			["ImprovedTraps"] = 343247,
			["Barrage"] = 120360,
			["Harpoon"] = 190925,
			["Ranger"] = 385695,
			["FuryoftheEagle"] = 203415,
			["TermsofEngagement"] = 265895,
			["ExplosivesExpert"] = 378937,
			["SentinelOwl"] = 388045,
			["KillShot"] = 320976,
			["SerratedShots"] = 389882,
			["RejuvenatingWind"] = 385539,
			["Ferocity"] = 378916,
			["TipoftheSpear"] = 260285,
			["FeignDeath"] = 5384,
			["BloodyClaws"] = 385737,
			["GuerrillaTactics"] = 264332,
			["EnergeticAlly"] = 378961,
			["RaptorStrike"] = 186270,
			["WildfireBomb"] = 259495,
			["DeadlyDuo"] = 378962,
			["EyesoftheBeast"] = 321297,
			["AspectoftheTurtle"] = 186265,
			["HydrasBite"] = 260241,
			["HuntersAvoidance"] = 384799,
			["ArcticBola"] = 390231,
			["AlphaPredator"] = 269737,
			["FlankersAdvantage"] = 263186,
			["ExplosiveShot"] = 212431,
			["BeastMaster"] = 378007,
			["CoordinatedKill"] = 385739,
			},
		},
		["Evoker"] = {
			["Devastation"] = {
			["FocusingIris"] = 386336,
			["TwinGuardian"] = 370888,
			["ArcaneIntensity"] = 375618,
			["ClobberingSweep"] = 375443,
			["Zephyr"] = 374227,
			["Tailwind"] = 375556,
			["EssenceAttunement"] = 375722,
			["ProtractedTalons"] = 369909,
			["ChargedBlast"] = 370455,
			["PowerSwell"] = 370839,
			["CauterizingFlame"] = 374251,
			["BlastFurnace"] = 375510,
			["Snapfire"] = 370783,
			["HoardedPower"] = 375796,
			["LayWaste"] = 371034,
			["ObsidianBulwark"] = 375406,
			["BlessingoftheBronze"] = 364342,
			["AncientFlame"] = 369990,
			["Iridescence"] = 370867,
			["InnerRadiance"] = 386405,
			["WallopingBlow"] = 387341,
			["Burnout"] = 375801,
			["EyeofInfinity"] = 411165,
			["DraconicLegacy"] = 376166,
			["FociofLife"] = 375574,
			["PermeatingChill"] = 370897,
			["PowerNexus"] = 369908,
			["Recall"] = 371806,
			["DeepBreath"] = 357210,
			["Firestorm"] = 368847,
			["TitanicWrath"] = 386272,
			["MasteryGiantkiller"] = 362980,
			["SleepWalk"] = 360806,
			["HeatWave"] = 375725,
			["NaturalConvergence"] = 369913,
			["RagingInferno"] = 405659,
			["Unravel"] = 368432,
			["EventHorizon"] = 411164,
			["AzureStrike"] = 362969,
			["LivingFlame"] = 361469,
			["Pyre"] = 357211,
			["FuryoftheAspects"] = 390386,
			["RenewingBlaze"] = 374348,
			["ObsidianScales"] = 363916,
			["Overawe"] = 374346,
			["ArcaneVigor"] = 386342,
			["FireBreath"] = 357208,
			["RubyEmbers"] = 365937,
			["DenseEnergy"] = 370962,
			["Landslide"] = 358385,
			["EternitysSpan"] = 375757,
			["InstinctiveArcana"] = 376164,
			["HonedAggression"] = 371038,
			["TemperedScales"] = 396571,
			["Rescue"] = 370665,
			["Causality"] = 375777,
			["Hover"] = 358267,
			["EternitySurge"] = 359073,
			["VerdantEmbrace"] = 360995,
			["EnergizingFlame"] = 400006,
			["Exuberance"] = 375542,
			["OnyxLegacy"] = 386348,
			["SourceofMagic"] = 369459,
			["TimeSpiral"] = 374968,
			["RegenerativeMagic"] = 387787,
			["ScarletAdaptation"] = 372469,
			["InnateMagic"] = 375520,
			["TerroroftheSkies"] = 371032,
			["FireWithin"] = 375577,
			["TiptheScales"] = 370553,
			["Enkindled"] = 375554,
			["LeapingFlames"] = 369939,
			["LushGrowth"] = 375561,
			["BountifulBloom"] = 370886,
			["Catalyze"] = 386283,
			["ExtendedFlight"] = 375517,
			["AzureEssenceBurst"] = 375721,
			["AerialMastery"] = 365933,
			["Volatility"] = 369089,
			["OppressingRoar"] = 372048,
			["ImposingPresence"] = 371016,
			["Dragonrage"] = 375087,
			["ForgerofMountains"] = 375528,
			["EmeraldBlossom"] = 355913,
			["Scintillation"] = 370821,
			["FontofMagic"] = 411212,
			["Panacea"] = 387761,
			["Expunge"] = 365585,
			["Tyranny"] = 376888,
			["RubyEssenceBurst"] = 376872,
			["EngulfingBlaze"] = 370837,
			["ShatteringStar"] = 370452,
			["Return"] = 361227,
			["Animosity"] = 375797,
			["Quell"] = 351338,
			["EverburningFlame"] = 370819,
			["PotentMana"] = 418101,
			["SpellweaversDominance"] = 370845,
			["InherentResistance"] = 375544,
			["AttunedtotheDream"] = 376930,
			["HeavyWingbeats"] = 368838,
			["FeedtheFlames"] = 369846,
			["ImminentDestruction"] = 370781,
			["Disintegrate"] = 356995,
			["EssenceBurst"] = 359565,
			},
			["Augmentation"] = {
			["SourceofMagic"] = 369459,
			["StretchTime"] = 410352,
			["MotesofPossibility"] = 409267,
			["TwinGuardian"] = 370888,
			["AspectsFavor"] = 407243,
			["ClobberingSweep"] = 375443,
			["Zephyr"] = 374227,
			["Tailwind"] = 375556,
			["InfernosBlessing"] = 410261,
			["EssenceAttunement"] = 375722,
			["EchoingStrike"] = 410784,
			["CauterizingFlame"] = 374251,
			["Prescience"] = 409311,
			["BlastFurnace"] = 375510,
			["PlottheFuture"] = 407866,
			["HoardedPower"] = 375796,
			["ObsidianBulwark"] = 375406,
			["BlessingoftheBronze"] = 364342,
			["AncientFlame"] = 369990,
			["LivingFlame"] = 361469,
			["InnerRadiance"] = 386405,
			["BlisteringScales"] = 360827,
			["FireBreath"] = 382266,
			["AzureStrike"] = 362969,
			["MasteryTimewalker"] = 406380,
			["ImposingPresence"] = 371016,
			["HeavyWingbeats"] = 368838,
			["FociofLife"] = 375574,
			["PermeatingChill"] = 370897,
			["PowerNexus"] = 369908,
			["AttunedtotheDream"] = 376930,
			["MomentumShift"] = 408004,
			["Recall"] = 371806,
			["DeepBreath"] = 357210,
			["Rescue"] = 370665,
			["RegenerativeChitin"] = 406907,
			["UnyieldingDomain"] = 412733,
			["NourishingSands"] = 406041,
			["InterwovenThreads"] = 412713,
			["EssenceBurst"] = 396187,
			["PotentMana"] = 418101,
			["SleepWalk"] = 360806,
			["ProlongLife"] = 410687,
			["NaturalConvergence"] = 369913,
			["Landslide"] = 358385,
			["Unravel"] = 368432,
			["PerilousFate"] = 410253,
			["ProtractedTalons"] = 369909,
			["EnergizingFlame"] = 400006,
			["FireWithin"] = 375577,
			["FuryoftheAspects"] = 390386,
			["RenewingBlaze"] = 374348,
			["ObsidianScales"] = 363916,
			["Overawe"] = 374346,
			["FateMirror"] = 412774,
			["DraconicLegacy"] = 376166,
			["RegenerativeMagic"] = 387787,
			["Overlord"] = 410260,
			["Eruption"] = 395160,
			["ScarletAdaptation"] = 372469,
			["Exuberance"] = 375542,
			["BestowWeyrnstone"] = 408233,
			["SandsofTime"] = 395153,
			["TerroroftheSkies"] = 371032,
			["SeismicSlam"] = 408543,
			["ReactiveHide"] = 409329,
			["Hover"] = 358267,
			["VerdantEmbrace"] = 360995,
			["DreamofSpring"] = 414969,
			["PupilofAlexstrasza"] = 407814,
			["Geomancy"] = 410787,
			["EbonMight"] = 395152,
			["TimeSpiral"] = 374968,
			["ChronoWard"] = 409676,
			["TiptheScales"] = 370553,
			["InnateMagic"] = 375520,
			["DefyFate"] = 404195,
			["Accretion"] = 407876,
			["Upheaval"] = 408092,
			["IgnitionRush"] = 408775,
			["Volcanism"] = 406904,
			["LushGrowth"] = 375561,
			["BountifulBloom"] = 370886,
			["SymbioticBloom"] = 410685,
			["ExtendedFlight"] = 375517,
			["InstinctiveArcana"] = 376164,
			["AerialMastery"] = 365933,
			["Expunge"] = 365585,
			["OppressingRoar"] = 372048,
			["BlackAttunement"] = 403264,
			["Enkindled"] = 375554,
			["ForgerofMountains"] = 375528,
			["EmeraldBlossom"] = 355913,
			["LeapingFlames"] = 369939,
			["FontofMagic"] = 408083,
			["CloseasClutchmates"] = 396043,
			["RicochetingPyroclast"] = 406659,
			["BreathofEons"] = 403631,
			["Panacea"] = 387761,
			["TimeSkip"] = 404977,
			["DraconicAttunements"] = 403208,
			["Return"] = 361227,
			["WallopingBlow"] = 387341,
			["Quell"] = 351338,
			["TemperedScales"] = 396571,
			["MoltenBlood"] = 410643,
			["SpatialParadox"] = 406732,
			["InherentResistance"] = 375544,
			["TomorrowToday"] = 412723,
			["Anachronism"] = 407869,
			["TectonicLocus"] = 408002,
			["Timelessness"] = 412710,
			["Disintegrate"] = 356995,
			["BronzeAttunement"] = 403265,
			},
			["Preservation"] = {
			["Stasis"] = 370537,
			["Erasure"] = 376210,
			["TwinGuardian"] = 370888,
			["Rescue"] = 370665,
			["GoldenHour"] = 378196,
			["Tailwind"] = 375556,
			["EssenceAttunement"] = 375722,
			["AttunedtotheDream"] = 376930,
			["EmeraldCommunion"] = 370960,
			["CauterizingFlame"] = 374251,
			["Ouroboros"] = 381921,
			["ExhilaratingBurst"] = 377100,
			["TimelessMagic"] = 376240,
			["ObsidianBulwark"] = 375406,
			["BlessingoftheBronze"] = 364342,
			["Lifebind"] = 373270,
			["AncientFlame"] = 369990,
			["LivingFlame"] = 361469,
			["WallopingBlow"] = 387341,
			["TimeLord"] = 372527,
			["Dreamwalker"] = 377082,
			["DraconicLegacy"] = 376166,
			["SpiritualClarity"] = 376150,
			["PermeatingChill"] = 370897,
			["PowerNexus"] = 369908,
			["Recall"] = 371806,
			["DeepBreath"] = 357210,
			["FireBreath"] = 382266,
			["Disintegrate"] = 356995,
			["EnergyLoop"] = 372233,
			["SourceofMagic"] = 369459,
			["LifeGiversFlame"] = 371426,
			["EssenceBurst"] = 369297,
			["PotentMana"] = 418101,
			["SleepWalk"] = 360806,
			["TemperedScales"] = 396571,
			["NaturalConvergence"] = 369913,
			["Landslide"] = 358385,
			["Unravel"] = 368432,
			["LeapingFlames"] = 369939,
			["FuryoftheAspects"] = 390386,
			["ObsidianScales"] = 363916,
			["FireWithin"] = 375577,
			["Empath"] = 376138,
			["RenewingBlaze"] = 374348,
			["MassReturn"] = 361178,
			["Overawe"] = 374346,
			["TemporalCompression"] = 362874,
			["FieldofDreams"] = 370062,
			["EnergizingFlame"] = 400006,
			["Hover"] = 358267,
			["Echo"] = 364343,
			["RenewingBreath"] = 371257,
			["Exuberance"] = 375542,
			["Spiritbloom"] = 382731,
			["CallofYsera"] = 373834,
			["RushofVitality"] = 377086,
			["ResonatingSphere"] = 376236,
			["TimeDilation"] = 357170,
			["TerroroftheSkies"] = 371032,
			["VerdantEmbrace"] = 360995,
			["RegenerativeMagic"] = 387787,
			["Panacea"] = 387761,
			["Zephyr"] = 374227,
			["ClobberingSweep"] = 375443,
			["TimeSpiral"] = 374968,
			["ScarletAdaptation"] = 372469,
			["Reversion"] = 366155,
			["InnateMagic"] = 375520,
			["Naturalize"] = 360823,
			["FociofLife"] = 375574,
			["LushGrowth"] = 375561,
			["Enkindled"] = 375554,
			["BountifulBloom"] = 370886,
			["GracePeriod"] = 376239,
			["DreamFlight"] = 359816,
			["AerialMastery"] = 365933,
			["ExtendedFlight"] = 375517,
			["Punctuality"] = 371270,
			["SparkofInsight"] = 377099,
			["Rewind"] = 363534,
			["OppressingRoar"] = 372048,
			["FlowState"] = 385696,
			["JustinTime"] = 376204,
			["ForgerofMountains"] = 375528,
			["EmeraldBlossom"] = 355913,
			["TiptheScales"] = 370553,
			["FontofMagic"] = 375783,
			["InstinctiveArcana"] = 376164,
			["Expunge"] = 365585,
			["BlastFurnace"] = 375510,
			["ProtractedTalons"] = 369909,
			["TimeofNeed"] = 368412,
			["MasteryLifeBinder"] = 363510,
			["Return"] = 361227,
			["LifeforceMender"] = 376179,
			["Quell"] = 351338,
			["TemporalArtificer"] = 381922,
			["CycleofLife"] = 371832,
			["NozdormusTeachings"] = 376237,
			["InherentResistance"] = 375544,
			["DreamBreath"] = 382614,
			["HeavyWingbeats"] = 368838,
			["FlutteringSeedlings"] = 359793,
			["AzureStrike"] = 362969,
			["TemporalAnomaly"] = 373861,
			["DelayHarm"] = 376207,
			},
		},
		["Death Knight"] = {
			["Unholy"] = {
			["MightofThassarian"] = 374111,
			["UnholyCommand"] = 316941,
			["ImprovedFesteringStrike"] = 316867,
			["UnholyAssault"] = 207289,
			["Assimilation"] = 374383,
			["AntiMagicZone"] = 51052,
			["DeathsReach"] = 276079,
			["ScourgeStrike"] = 55090,
			["UnholyBond"] = 374261,
			["MindFreeze"] = 47528,
			["RuneMastery"] = 374574,
			["RupturedViscera"] = 390236,
			["WilloftheNecropolis"] = 206967,
			["SoulReaper"] = 343294,
			["ChainsofIce"] = 45524,
			["AbominationLimb"] = 383269,
			["ImprovedDeathCoil"] = 377580,
			["MarchofDarkness"] = 391546,
			["DarkCommand"] = 56222,
			["Epidemic"] = 207317,
			["AntiMagicShell"] = 48707,
			["ClawingShadows"] = 207311,
			["UnholyEndurance"] = 389682,
			["CommanderoftheDead"] = 390259,
			["InfectedClaws"] = 207272,
			["ControlUndead"] = 111673,
			["ProliferatingChill"] = 373930,
			["WraithWalk"] = 212552,
			["RunicCorruption"] = 51462,
			["MasteryDreadblade"] = 77515,
			["IcyTalons"] = 194878,
			["DarkTransformation"] = 63560,
			["Brittle"] = 374504,
			["IceboundFortitude"] = 48792,
			["SacrificialPact"] = 327574,
			["AntiMagicBarrier"] = 205727,
			["Acclimation"] = 373926,
			["EmpowerRuneWeapon"] = 47568,
			["Outbreak"] = 77575,
			["DeathCoil"] = 47541,
			["MercilessStrikes"] = 373923,
			["InsidiousChill"] = 391566,
			["UnholyGround"] = 374265,
			["Permafrost"] = 207200,
			["ReplenishingWounds"] = 377585,
			["EternalAgony"] = 390268,
			["EbonFever"] = 207269,
			["DeathsEcho"] = 356367,
			["FesteringStrike"] = 85948,
			["BlindingSleet"] = 207167,
			["VileContagion"] = 390279,
			["ImprovedDeathStrike"] = 374277,
			["AllWillServe"] = 194916,
			["DeathRot"] = 377537,
			["RottenTouch"] = 390275,
			["ArmyoftheDamned"] = 276837,
			["ArmyoftheDead"] = 42650,
			["GripoftheDead"] = 273952,
			["Festermight"] = 377590,
			["UnholyAura"] = 377440,
			["DeathGrip"] = 49576,
			["RunicMastery"] = 390166,
			["DeathStrike"] = 49998,
			["UnholyBlight"] = 115989,
			["CleavingStrikes"] = 316916,
			["DeathPact"] = 48743,
			["Superstrain"] = 390283,
			["VeteranoftheThirdWar"] = 48263,
			["OnaPaleHorse"] = 51986,
			["SummonGargoyle"] = 49206,
			["Enfeeble"] = 392566,
			["Coldthirst"] = 378848,
			["GhoulishFrenzy"] = 377587,
			["Runeforging"] = 53428,
			["PathofFrost"] = 3714,
			["UnholyPact"] = 319230,
			["Defile"] = 152280,
			["RaiseDead"] = 46584,
			["Pestilence"] = 277234,
			["BloodDraw"] = 374598,
			["RuneStrike"] = 316239,
			["RunicAttenuation"] = 207104,
			["Suppression"] = 374049,
			["RaiseAlly"] = 61999,
			["DarkSuccor"] = 178819,
			["ClenchingGrasp"] = 389679,
			["SuddenDoom"] = 49530,
			["Plaguebringer"] = 390175,
			["Lichborne"] = 49039,
			["Morbidity"] = 377592,
			["Reaping"] = 377514,
			["DeathGate"] = 50977,
			["BloodScent"] = 374030,
			["DeathandDecay"] = 43265,
			["CoilofDevastation"] = 390270,
			["MagusoftheDead"] = 390196,
			["HarbingerofDoom"] = 276023,
			["FeastingStrikes"] = 390161,
			["GloomWard"] = 391571,
			["Asphyxiate"] = 221562,
			["DeathsAdvance"] = 48265,
			["BurstingSores"] = 207264,
			["Apocalypse"] = 275699,
			},
			["Frost"] = {
			["MightofThassarian"] = 374111,
			["ImprovedFrostStrike"] = 316803,
			["Assimilation"] = 374383,
			["InvigoratingFreeze"] = 377092,
			["AntiMagicZone"] = 51052,
			["DeathsReach"] = 276079,
			["RunicCommand"] = 376251,
			["FrostwhelpsAid"] = 377226,
			["UnholyBond"] = 374261,
			["ColdBloodedRage"] = 377083,
			["ShatteringBlade"] = 207057,
			["MindFreeze"] = 47528,
			["RuneMastery"] = 374574,
			["DeathsAdvance"] = 48265,
			["WilloftheNecropolis"] = 206967,
			["SoulReaper"] = 343294,
			["ChainsofIce"] = 45524,
			["SacrificialPact"] = 327574,
			["FrostwyrmsFury"] = 279302,
			["MarchofDarkness"] = 391546,
			["ImprovedRime"] = 316838,
			["FrigidExecutioner"] = 377073,
			["AntiMagicShell"] = 48707,
			["MightoftheFrozenWastes"] = 81333,
			["UnholyEndurance"] = 389682,
			["Everfrost"] = 376938,
			["UnleashedFrenzy"] = 376905,
			["ImprovedObliterate"] = 317198,
			["ControlUndead"] = 111673,
			["PiercingChill"] = 377351,
			["AbsoluteZero"] = 377047,
			["ProliferatingChill"] = 373930,
			["WraithWalk"] = 212552,
			["PillarofFrost"] = 51271,
			["MurderousEfficiency"] = 207061,
			["IceboundFortitude"] = 48792,
			["RunicEmpowerment"] = 81229,
			["MasteryFrozenHeart"] = 77514,
			["Bonegrinder"] = 377098,
			["MercilessStrikes"] = 373923,
			["EmpowerRuneWeapon"] = 47568,
			["FrostStrike"] = 49143,
			["Coldthirst"] = 378848,
			["GloomWard"] = 391571,
			["DeathandDecay"] = 43265,
			["InexorableAssault"] = 253593,
			["InsidiousChill"] = 391566,
			["EnduringChill"] = 377376,
			["BlindingSleet"] = 207167,
			["RageoftheFrozenChampion"] = 377076,
			["AbominationLimb"] = 383269,
			["IcyTalons"] = 194878,
			["DeathCoil"] = 47541,
			["Rime"] = 59057,
			["HornofWinter"] = 57330,
			["Avalanche"] = 207142,
			["GripoftheDead"] = 273952,
			["BitingCold"] = 377056,
			["Permafrost"] = 207200,
			["DeathGrip"] = 49576,
			["Obliterate"] = 49020,
			["DeathStrike"] = 49998,
			["Frostscythe"] = 207230,
			["HowlingBlast"] = 49184,
			["DeathPact"] = 48743,
			["OnaPaleHorse"] = 51986,
			["ImprovedDeathStrike"] = 374277,
			["DarkSuccor"] = 178819,
			["ChillStreak"] = 305392,
			["Enfeeble"] = 392566,
			["UnholyGround"] = 374265,
			["ClenchingGrasp"] = 389679,
			["RaiseDead"] = 46585,
			["PathofFrost"] = 3714,
			["Icebreaker"] = 392950,
			["RunicAttenuation"] = 207104,
			["Runeforging"] = 53428,
			["Icecap"] = 207126,
			["BloodDraw"] = 374598,
			["RuneStrike"] = 316239,
			["EnduringStrength"] = 377190,
			["Suppression"] = 374049,
			["VeteranoftheThirdWar"] = 48263,
			["RaiseAlly"] = 61999,
			["GlacialAdvance"] = 194913,
			["BreathofSindragosa"] = 152279,
			["GatheringStorm"] = 194912,
			["Lichborne"] = 49039,
			["Obliteration"] = 281238,
			["ColdHeart"] = 281208,
			["DeathGate"] = 50977,
			["BloodScent"] = 374030,
			["RemorselessWinter"] = 196770,
			["DeathsEcho"] = 356367,
			["KillingMachine"] = 51128,
			["AntiMagicBarrier"] = 205727,
			["DarkCommand"] = 56222,
			["Brittle"] = 374504,
			["Asphyxiate"] = 221562,
			["CleavingStrikes"] = 316916,
			["Acclimation"] = 373926,
			["FatalFixation"] = 405166,
			},
			["Blood"] = {
			["MightofThassarian"] = 374111,
			["DeathsCaress"] = 195292,
			["Ossuary"] = 219786,
			["Assimilation"] = 374383,
			["BloodBoil"] = 50842,
			["BloodTap"] = 221699,
			["AntiMagicZone"] = 51052,
			["DeathsReach"] = 276079,
			["UnholyBond"] = 374261,
			["MindFreeze"] = 47528,
			["RuneMastery"] = 374574,
			["DeathsAdvance"] = 48265,
			["WilloftheNecropolis"] = 206967,
			["UmbilicusEternus"] = 391517,
			["ChainsofIce"] = 45524,
			["SacrificialPact"] = 327574,
			["MarchofDarkness"] = 391546,
			["DarkCommand"] = 56222,
			["AntiMagicShell"] = 48707,
			["Marrowrend"] = 195182,
			["UnholyEndurance"] = 389682,
			["Bonestorm"] = 194844,
			["BloodFeast"] = 391386,
			["ControlUndead"] = 111673,
			["BloodFortification"] = 374721,
			["ProliferatingChill"] = 373930,
			["WraithWalk"] = 212552,
			["VampiricBlood"] = 55233,
			["AbominationLimb"] = 383269,
			["ImprovedHeartStrike"] = 374717,
			["Tombstone"] = 219809,
			["CrimsonScourge"] = 81136,
			["IceboundFortitude"] = 48792,
			["Blooddrinker"] = 206931,
			["GloomWard"] = 391571,
			["DancingRuneWeapon"] = 49028,
			["GorefiendsGrasp"] = 108199,
			["IcyTalons"] = 194878,
			["InsidiousChill"] = 391566,
			["MercilessStrikes"] = 373923,
			["SoulReaper"] = 343294,
			["CleavingStrikes"] = 316916,
			["MarkofBlood"] = 206940,
			["Permafrost"] = 207200,
			["DeathandDecay"] = 43265,
			["Voracious"] = 273953,
			["DeathsEcho"] = 356367,
			["FoulBulwark"] = 206974,
			["BlindingSleet"] = 207167,
			["HeartStrike"] = 206930,
			["ImprovedVampiricBlood"] = 317133,
			["ImprovedDeathStrike"] = 374277,
			["LeechingStrike"] = 377629,
			["UnholyGround"] = 374265,
			["Purgatory"] = 114556,
			["ClenchingGrasp"] = 389679,
			["GripoftheDead"] = 273952,
			["PerseveranceoftheEbonBlade"] = 374747,
			["Coagulopathy"] = 391477,
			["InsatiableBlade"] = 377637,
			["ReinforcedBones"] = 374737,
			["DeathStrike"] = 49998,
			["EverlastingBond"] = 377668,
			["VeteranoftheThirdWar"] = 48263,
			["DeathPact"] = 48743,
			["Bloodworms"] = 195679,
			["RuneTap"] = 194679,
			["OnaPaleHorse"] = 51986,
			["Consumption"] = 274156,
			["Enfeeble"] = 392566,
			["Heartrend"] = 377655,
			["RaiseDead"] = 46585,
			["Heartbreaker"] = 221536,
			["PathofFrost"] = 3714,
			["BloodDraw"] = 374598,
			["RedThirst"] = 205723,
			["Runeforging"] = 53428,
			["RunicAttenuation"] = 207104,
			["SanguineGround"] = 391458,
			["RuneStrike"] = 316239,
			["RapidDecomposition"] = 194662,
			["Suppression"] = 374049,
			["RaiseAlly"] = 61999,
			["MasteryBloodShield"] = 77513,
			["TighteningGrasp"] = 206970,
			["IronHeart"] = 391395,
			["Hemostasis"] = 273946,
			["Lichborne"] = 49039,
			["DeathGrip"] = 49576,
			["Acclimation"] = 373926,
			["DeathGate"] = 50977,
			["BloodScent"] = 374030,
			["DeathCoil"] = 47541,
			["Bloodshot"] = 391398,
			["EmpowerRuneWeapon"] = 47568,
			["AntiMagicBarrier"] = 205727,
			["RelishinBlood"] = 317610,
			["Brittle"] = 374504,
			["Asphyxiate"] = 221562,
			["ImprovedBoneShield"] = 374715,
			["Coldthirst"] = 378848,
			["ShatteringBone"] = 377640,
			},
		},
		}
	local className, classFilename, classId = UnitClass("player")
	local currentSpec = GetSpecialization()
	local id, name, description, icon, background, role = GetSpecializationInfo(currentSpec)
	MaxDps.SpellTable = MaxDpsSpellTable[idtoclass[classId]][idtospec[id]]
	--MaxDps.SpellInfoTable = {}
end

function MaxDps:UNIT_ENTERED_VEHICLE(_, unit)
	if unit == 'player' and self.rotationEnabled then
		self:DisableRotation();
	end
end

function MaxDps:UNIT_EXITED_VEHICLE(_, unit)
	if unit == 'player' then
		self:InitRotations();
		self:EnableRotation();
	end
end

function MaxDps:PLAYER_TARGET_CHANGED()
	if self.rotationEnabled then
		if UnitIsFriend('player', 'target') then
			return
		else
			self:InvokeNextSpell();
		end
	end
end

function MaxDps:PLAYER_REGEN_DISABLED()
	if self.db.global.onCombatEnter and not self.rotationEnabled then
		self:Print(self.Colors.Success .. 'Auto enable on combat!', "info");
		self:InitRotations();
		self:EnableRotation();
	end
end

function MaxDps:PLAYER_REGEN_ENABLED()
	self:DisableRotation();
end

function MaxDps:ButtonFetch()
	if self.rotationEnabled then
		if self.fetchTimer then
			self:CancelTimer(self.fetchTimer);
		end
		self.fetchTimer = self:ScheduleTimer('Fetch', 0.5);
	end
end

function MaxDps:PrepareFrameData()
	if not self.FrameData then
		self.FrameData = {
			cooldown  = self.PlayerCooldowns,
			activeDot = self.ActiveDots
		};
	end

	self.FrameData.timeShift, self.FrameData.currentSpell, self.FrameData.gcdRemains = MaxDps:EndCast();
	self.FrameData.gcd = self:GlobalCooldown();
	self.FrameData.buff, self.FrameData.debuff = MaxDps:CollectAuras();
	self.FrameData.talents = self.PlayerTalents;
	self.FrameData.azerite = self.AzeriteTraits;
	self.FrameData.essences = self.AzeriteEssences;
	self.FrameData.covenant = self.CovenantInfo;
	self.FrameData.runeforge = self.LegendaryBonusIds;
	self.FrameData.spellHistory = self.spellHistory;
	self.FrameData.timeToDie = self:GetTimeToDie();
end

function MaxDps:InvokeNextSpell()
	-- invoke spell check
	local oldSkill = self.Spell;

	self:PrepareFrameData();

	self:GlowConsumables();

	-- Removed backward compatibility
	self.Spell = self:NextSpell();

	if (oldSkill ~= self.Spell or oldSkill == nil) and self.Spell ~= nil then
		self:GlowNextSpell(self.Spell);
		if WeakAuras then
			WeakAuras.ScanEvents('MAXDPS_SPELL_UPDATE', self.Spell);
		end
	end

	if self.Spell == nil and oldSkill ~= nil then
		self:GlowClear();
		if WeakAuras then
			WeakAuras.ScanEvents('MAXDPS_SPELL_UPDATE', nil);
		end
	end
end

function MaxDps:InitRotations()
	self:Print(self.Colors.Info .. 'Initializing rotations', "info");
	self:CountTier()

	local _, _, classId = UnitClass('player');
	local spec = GetSpecialization();
	self.ClassId = classId;
	self.Spec = spec;

	if not self.Custom then
		self.Custom = self:GetModule('Custom');
	end

	self.Custom:LoadCustomRotations();
	local customRotation = self.Custom:GetCustomRotation(classId, spec);

	if customRotation then
		self.NextSpell = customRotation.fn;

		self:Print(self.Colors.Success .. 'Loaded Custom Rotation: ' .. customRotation.name, "info");
	else
		self:LoadModule();
	end
end

function MaxDps:LoadModule()
	if self.Classes[self.ClassId] == nil then
		self:Print(self.Colors.Error .. 'Invalid player class, please contact author of addon.', "error");
		return
	end

	local className = self.Classes[self.ClassId];
	local module = 'MaxDps_' .. className;
	local _, _, _, _, reason = GetAddOnInfo(module);

	if IsAddOnLoaded(module) then
		self:EnableRotationModule(className);
		return
	end

	if reason == 'MISSING' or reason == 'DISABLED' then
		self:Print(self.Colors.Error .. 'Could not find class module ' .. module .. ', reason: ' .. reason, "error");
		self:Print(self.Colors.Error .. 'Make sure to install class module or create custom rotation', "error");
		self:Print(self.Colors.Error .. 'Missing addon: ' .. module, "error");
		return
	end

	LoadAddOn(module);

	self:InitTTD();
	self:EnableRotationModule(className);
end

function MaxDps:EnableRotationModule(className)
	local loaded = self:EnableModule(className);

	if not loaded then
		self:Print(self.Colors.Error .. 'Could not find load module ' .. className .. ', reason: OUTDATED', "error");
	else
		self:Print(self.Colors.Info .. 'Finished Loading class module', "info");
	end
end
